"""Loads all necessary data
"""
import os
import csv
import logging
import configparser
import ast
from collections import defaultdict
import numpy as np
import pandas as pd
import geopandas as gpd
from shapely.geometry import Point
import matplotlib.pyplot as plt

from energy_demand.basic import lookup_tables
from energy_demand.read_write import write_data
from energy_demand.read_write import read_data, read_weather_data
from energy_demand.basic import conversions
from energy_demand.plotting import fig_lp
from energy_demand.basic import basic_functions
from energy_demand.read_write import narrative_related

def create_weather_station_map(stations_as_dict,
                               fig_path,
                               path_shapefile=False):
    """Plot the spatial distribution of the weather stations

    https://geopandas.readthedocs.io/en/latest/gallery/create_geopandas_from_pandas.html

    df = pd.DataFrame(
        {'src_id': [...],
        'longitude': [...],
        'latitude': [...]})
    """
    if stations_as_dict == {}:
        print("No stations available to plot")
    else:
        # Convert dict to dataframe
        df = pd.DataFrame.from_dict(stations_as_dict, orient='index')

        df['Coordinates'] = list(zip(df.longitude, df.latitude))
        df['Coordinates'] = df['Coordinates'].apply(Point)

        if path_shapefile is False:
            world = gpd.read_file(gpd.datasets.get_path('naturalearth_lowres'))
            ax = world[world.name == "United Kingdom"].plot(
                color='white', edgecolor='black')
        else:
            # Load uk shapefile
            uk_shapefile = gpd.read_file(path_shapefile)

            # Assign correct projection
            crs = {'init': 'epsg:27700'} #27700 == OSGB_1936_British_National_Grid
            uk_gdf = gpd.GeoDataFrame(uk_shapefile, crs=crs)

            # Transform
            uk_gdf = uk_gdf.to_crs({'init': 'epsg:4326'})

            # Plot
            ax = uk_gdf.plot(color='white', edgecolor='black')

        # print coordinates
        crs = {'init': 'epsg:4326'}
        gdf = gpd.GeoDataFrame(df, geometry='Coordinates') # crs=crs,
        gdf.plot(ax=ax, color='red')

        plt.savefig(fig_path)

        # ---------------------------------
        # Save coordinates as file
        # ---------------------------------
        station_infos = ["name, latitue, longitude"]
        fig_path = fig_path[:-4] + ".txt"
        for station in stations_as_dict:
            station_info = "{}, {}, {}".format(
                station,
                stations_as_dict[station]['latitude'],
                stations_as_dict[station]['longitude'])
            station_infos.append(station_info)

        write_data.write_list_to_txt(fig_path, station_infos)


def read_weather_stations_raw(path_to_csv):
    """Read in weather stations from csv file

    Parameter
    ---------
    path_to_csv : string
        Path to csv with stored weater station data

    Returns:
    --------
    weather_stations : dict
        Contains coordinates and station_id of weather stations

    Note
    ----
    Downloaded from MetOffice
    http://archive.ceda.ac.uk/cgi-bin/midas_stations/search_by_name.cgi.py?name=&minyear=&maxyear=&current=n&db=midas_stations&orderby=id (21-09-2018)
    """
    df_stations = pd.read_csv(path_to_csv)

    weather_stations = {}

    for _, row in df_stations.iterrows():

        # Filter out not fitting weather stations
        if float(row['Longitude']) < -8:
            pass
        elif float(row['Latitude']) < 50:
            pass
        else:
            weather_stations[int(row['src_id'])] = {
                'latitude' : float(row['Latitude']), #TODO CHANGE TO 'ltatitude
                'longitude': float(row['Longitude'])} #TODO CHANGE TO 'ltatitude

    return weather_stations

def replace_variable(_user_defined_vars, strategy_vars):
    """Replace default strategy variables with user defined variables

    Arguments
    ----------
    _user_defined_vars : dict
        User defined strategy variables
    strategy_vars : dict
        Default strategy variables

    Returns
    -------
    strategy_vars : dict
        Updated strategy variables with user defined inputs
    """
    for new_var, new_var_vals in _user_defined_vars.items():

        crit_single_dim = narrative_related.crit_dim_var(new_var_vals)

        if crit_single_dim:
            strategy_vars[new_var] = new_var_vals
        else:
            for sub_var_name, sector_sub_var in new_var_vals.items():

                if type(sector_sub_var) is dict:
                    strategy_vars[new_var][sub_var_name] = {}
                    for sector, sub_var in sector_sub_var.items():
                        strategy_vars[new_var][sub_var_name][sector] = sub_var
                else:
                    strategy_vars[new_var][sub_var_name] = sector_sub_var

            strategy_vars[new_var] = strategy_vars[new_var]
    return strategy_vars

def load_local_user_defined_vars(
        default_strategy_var,
        path_csv,
        simulation_base_yr,
        simulation_end_yr
    ):
    """Load all strategy variables from file

    Arguments
    ---------
    default_strategy_var : dict
        default strategy var
    path_csv : str
        Path to folder with all user defined parameters
    simulation_base_yr : int
        Simulation base year

    Returns
    -------
    strategy_vars_as_narratives : dict
        Single or multidimensional parameters with fully autocompleted narratives
    """
    all_csv_in_folder = os.listdir(path_csv)

    # Files to ignore in this folder
    files_to_ignores = [
        'switches_capacity.csv',
        'switches_fuel.csv',
        'switches_service.csv',
        '_README_config_data.txt']

    strategy_vars_as_narratives = {}

    for file_name in all_csv_in_folder:
        print("... filename: " + str(file_name))
        if file_name not in files_to_ignores:

            var_name = file_name[:-4] #remove ".csv"

            raw_file_content = pd.read_csv(os.path.join(path_csv, file_name))

            strategy_vars_as_narratives[var_name] = narrative_related.read_user_defined_param(
                raw_file_content,
                simulation_base_yr=simulation_base_yr,
                simulation_end_yr=simulation_end_yr,
                default_streategy_var=default_strategy_var[var_name],
                var_name=var_name)

    return strategy_vars_as_narratives

def load_ini_param(path):
    """Load simulation parameter run information

    Arguments
    ---------
    path : str
        Path to `ini` file

    Returns
    -------
    enduses : dict
        Enduses
    assumptions : dict
        Assumptions
    reg_nrs : dict
        Number of regions
    regions : dict
        Regions
    """
    config = configparser.ConfigParser()
    config.read(os.path.join(path, 'model_run_sim_param.ini'))

    regions = ast.literal_eval(config['REGIONS']['regions'])

    assumptions = {}
    assumptions['reg_nrs'] = int(config['SIM_PARAM']['reg_nrs'])
    assumptions['base_yr'] = int(config['SIM_PARAM']['base_yr'])
    assumptions['sim_yrs'] = ast.literal_eval(config['SIM_PARAM']['sim_yrs'])

    # -----------------
    # Other information
    # -----------------
    enduses = {}
    enduses['residential'] = ast.literal_eval(config['ENDUSES']['residential'])
    enduses['service'] = ast.literal_eval(config['ENDUSES']['service'])
    enduses['industry'] = ast.literal_eval(config['ENDUSES']['industry'])

    return enduses, assumptions, regions

def load_MOSA_pop(path_to_csv):
    """
    Load MPSA population
    """
    pop_data = defaultdict(dict)

    with open(path_to_csv, 'r') as csvfile:
        rows = csv.reader(csvfile, delimiter=',')
        headings = next(rows)

        for row in rows:
            lad_code = str.strip(row[read_data.get_position(headings, 'Local authority code')])
            MSOA_code = row[read_data.get_position(headings, 'MSOA Code')].strip()
            pop = float(row[read_data.get_position(headings, 'Persons')].strip().replace(",", ""))

            pop_data[lad_code][MSOA_code] = pop

    return pop_data

def read_lad_demands(path_to_csv):
    """Read in national consumption from csv file. The unit
    in the original csv is in GWh per region per year.

    Arguments
    ---------
    path_to_csv : str
        Path to csv file

    Returns
    -------
    national_fuel_data : dict
        geocode, total consumption

    Source
    -----
    https://www.gov.uk/government/statistical-data-sets
    /regional-and-local-authority-gas-
    consumption-statistics-2005-to-2011
    https://www.gov.uk/government/statistical-data-sets
    /regional-and-local-authority-electricity-
    consumption-statistics-2005-to-2011
    """
    national_fuel_data = {}
    with open(path_to_csv, 'r') as csvfile:
        rows = csv.reader(csvfile, delimiter=',')
        headings = next(rows)

        for row in rows:
            geocode = str.strip(row[read_data.get_position(headings, 'LA Code')])
            tot_consumption_unclean = row[read_data.get_position(headings, 'Total consumption')].strip()

            if tot_consumption_unclean == '-':
                pass
            else:
                try:
                    national_fuel_data[geocode] = float(tot_consumption_unclean.replace(",", ""))
                except:
                    # no data provided
                    logging.debug("No validation data available for region %s", geocode)

    return national_fuel_data

def read_elec_data_msoa(path_to_csv):
    """Read in msoa consumption from csv file. The unit
    in the original csv is in kWh per region per year.

    Arguments
    ---------
    path_to_csv : str
        Path to csv file

    Returns
    -------
    national_fuel_data : dict
        geocode, total consumption

    Info
    -----
    Source: https://www.gov.uk/government/statistical-data-sets
    /regional-and-local-authority-electricity-
    consumption-statistics-2005-to-2011
    """
    national_fuel_data = {}
    with open(path_to_csv, 'r') as csvfile:
        rows = csv.reader(csvfile, delimiter=',')
        headings = next(rows)

        for row in rows:
            geocode = str.strip(row[read_data.get_position(headings, 'msoa_code')])
            tot_consumption_unclean = row[read_data.get_position(headings, 'tot_conump_kWh')].strip()
            national_fuel_data[geocode] = float(tot_consumption_unclean.replace(",", ""))

    return national_fuel_data

def floor_area_virtual_dw(
        regions,
        sectors,
        local_paths,
        population,
        base_yr,
        f_mixed_floorarea=0.5
    ):
    """Load necessary data for virtual building stock
    in case the link to the building stock model in
    Newcastle is not used

    Arguments
    ---------
    regions : dict
        Regions
    sectors : dict
        All sectors
    local_paths : dict
        Paths
    base_yr : float
        Base year
    f_mixed_floorarea : float
        PArameter to redistributed mixed enduse
    regions_without_floorarea : float
        Regions with missing floor area info
    Returns
    -------
    rs_floorarea : dict
        Residential floor area
    ss_floorarea : dict
        Service sector floor area
    """
    # ------
    # Get average floor area per perons
    # Based on Roberts et al. (2011) , an average one bedroom home for 2 people has 46 m2.
    # Roberts et al. (2011): The Case for Space: the size of England’s new homes.
    # -----
    rs_avearge_floor_area_pp = 23   # [m2] Assumed average residential area per person
    ss_avearge_floor_area_pp = 23   # [m2] Assumed average service area per person

    # --------------------------------------------------
    # Floor area for residential buildings for base year
    # from newcasle dataset
    # --------------------------------------------------
    resid_footprint, non_res_flootprint, service_building_count = read_data.read_floor_area_virtual_stock(
        local_paths['path_floor_area_virtual_stock_by'],
        f_mixed_floorarea=f_mixed_floorarea)

    # -----------------
    # Calculate average floor area per person
    # of existing datasets. This is done to replace the missing
    # floor area data of LADs with estimated floor areas
    # -----------------
    rs_regions_without_floorarea = []
    rs_floorarea = defaultdict(dict)
    for region in regions:
        try:
            rs_floorarea[base_yr][region] = resid_footprint[region]
        except KeyError:
            ##print("No virtual residential floor area for region %s ", region)

            # Calculate average floor area
            rs_floorarea[base_yr][region] = rs_avearge_floor_area_pp * population[region]
            rs_regions_without_floorarea.append(region)

    # --------------------------------------------------
    # Floor area for service sector buildings
    # --------------------------------------------------
    ss_floorarea_sector_by = {}
    ss_regions_without_floorarea = set([])
    ss_floorarea_sector_by[base_yr] = defaultdict(dict)
    for region in regions:
        for sector in sectors['service']:
            try:
                ss_floorarea_sector_by[base_yr][region][sector] = non_res_flootprint[region]
            except KeyError:

                #logging.debug("No virtual service floor area for region %s", region)

                # Calculate average floor area if no data is available
                ss_floor_area_cy = ss_avearge_floor_area_pp * population[region]
                ss_floorarea_sector_by[base_yr][region][sector] = ss_floor_area_cy
                ss_regions_without_floorarea.add(region)

    return dict(rs_floorarea), dict(ss_floorarea_sector_by), service_building_count, rs_regions_without_floorarea, list(ss_regions_without_floorarea)

def read_config_file(config_file_path):
    """Reads all sections of a configuration file

    Arguments
    ---------
    config_file_path : str
        Absolute path to the configuration file

    Returns
    -------
    dict
        A nested dictionary containing all the paths

    """
    config = configparser.ConfigParser()
    config.read(config_file_path)

    return convert_config_to_correct_type(config)

def convert_config_to_correct_type(config):
    """Convert config types into correct types
    """
    out_dict = defaultdict(dict)

    string_sections = ['PATHS', 'DATA_PATHS', 'CONFIG_DATA']
    for section in string_sections:
        for path in config[section]:
            out_dict[section][path] = config.get(section, path)

    for config_section in config['CONFIG']:
        out_dict['CONFIG'][config_section] = config.getint('CONFIG', config_section)

    for criteria in config['CRITERIA']:
        if criteria == 'reg_selection_csv_name':
            out_dict['CRITERIA'][criteria] = config.get('CRITERIA', criteria)
        else:
            out_dict['CRITERIA'][criteria] = config.getboolean('CRITERIA', criteria)

    return dict(out_dict)

def get_local_paths(config_file_path):
    """Create all local paths

    Local paths with data used for model config, raw data and process data from
    the `DATA_PATHS` section of the config file

    Arguments
    --------
    config_file_path : str
        config_file_path to the wrapperconfig.ini configuration file

    Return
    -------
    data_paths : dict
        All local paths used in model
    """
    paths = [
        ('local_path_datafolder', 'str'),
        ('path_strategy_vars', 'str'),
        ('path_population_data_for_disaggregation_LAD', 'str'),
        ('path_population_data_for_disaggregation_MSOA', 'str'),
        ('folder_raw_carbon_trust', 'str'),
        ('folder_path_weater_stations', 'str'),
        ('path_floor_area_virtual_stock_by', 'str'),
        ('path_assumptions_db', 'str'),
        ('data_processed', 'str'),
        ('lad_shapefile', 'str'),
        ('path_post_installation_data', 'str'),
        ('weather_data', 'str'),
        ('load_profiles', 'str'),
        ('rs_load_profile_txt', 'str'),
        ('ss_load_profile_txt', 'str'),
        ('yaml_parameters', 'str'),
        ('yaml_parameters_constrained', 'str'),
        ('yaml_parameters_keynames_constrained', 'str'),
        ('yaml_parameters_keynames_unconstrained', 'str'),
        ('yaml_parameters_scenario', 'str')]

    return _read_config_file_section('DATA_PATHS', config_file_path, paths)


def _read_config_file_section(section, config_file_path, items):
    """Read a configuration file section

    Arguments
    ---------
    section : str
        Name of the section to read
    config_file_path : str
        Absolute path to the config file
    items : list of str
        A list of names of the config items to read

    Returns
    -------
    dict
        Keys are the item names

    Raises
    ------
    ValueError if the configuration item does not exist in the file
    """
    data_paths = {}
    config = configparser.ConfigParser()
    config.read(config_file_path)
    for item, item_type in items:

        try:
            if item_type == 'str':
                data_paths[item] = config.get(section, item)
            elif item_type == 'bool':
                data_paths[item] = config.getboolean(section, item)
            elif item_type == 'int':
                data_paths[item] = config.getint(section, item)

        except configparser.NoOptionError:
            msg = "Option '{}' doesn't exist in section '{}' in '{}'"
            raise ValueError(msg.format(item, section, config_file_path))
        except configparser.NoSectionError:
            msg = "Section '{}' doesn't exist in '{}'"
            raise ValueError(msg.format(section, config_file_path))

    return data_paths

'''def get_result_paths(config_file_path):
    """Load all result paths from the `RESULT_DATA` section of the config file

    Arguments
    --------
    config_file_path : str
        config_file_path to the wrapperconfig.ini configuration file

    Return
    -------
    paths : dict
        All result paths used in model
    """
    paths = [
        ('data_results_model_run_pop', 'str'),
        ('data_results_model_runs', 'str'),
        ('data_results_PDF', 'str'),
        ('data_results_validation', 'str'),
        ('data_results_shapefiles', 'str'),
        ('individual_enduse_lp', 'str')]

    return _read_config_file_section('RESULT_DATA', config_file_path, paths)'''

def load_paths(config_file_path):
    """Load all paths from the `CONFIG_DATA` of the config file

    Arguments
    ----------
    config_file_path : str
        Main path

    Return
    ------
    dict
        Dictionary containing the paths to each datafile
    """
    paths = [
        ('path_main', 'str'),
        # Path to all technologies
        ('path_technologies', 'str'),
        # Paths to fuel raw data
        ('rs_fuel_raw', 'str'),
        ('ss_fuel_raw', 'str'),
        ('is_fuel_raw', 'str'),
        # Load profiles
        ('lp_rs', 'str'),
        # Technologies load shapes
        ('path_hourly_gas_shape_resid', 'str'),
        ('lp_elec_hp_dh', 'str'),
        ('lp_all_microCHP_dh', 'str'),
        ('path_shape_rs_cooling', 'str'),
        ('path_shape_ss_cooling', 'str'),
        ('lp_elec_storage_heating', 'str'),
        ('lp_elec_secondary_heating', 'str'),
        # Census data
        ('path_employment_statistics', 'str'),
        # Validation datasets
        ('val_subnational_elec', 'str'),
        ('val_subnational_elec_residential', 'str'),
        ('val_subnational_elec_non_residential', 'str'),
        ('val_subnational_elec_msoa_residential', 'str'),
        ('val_subnational_elec_msoa_non_residential', 'str'),
        ('val_subnational_gas', 'str'),
        ('val_subnational_gas_residential', 'str'),
        ('val_subnational_gas_non_residential', 'str'),
        ('val_nat_elec_data', 'str')]

    return _read_config_file_section('CONFIG_DATA', config_file_path, paths)


def load_tech_profiles(
        tech_lp,
        paths,
        local_paths,
        plot_tech_lp=True
    ):
    """Load technology specific load profiles

    Arguments
    ----------
    tech_lp : dict
        Load profiles
    paths : dict
        Path to open config data
    local_paths : dict
        Local data paths (private data downloaded from ftp)
    plot_tech_lp : bool
        Criteria whether individual tech lp are saved as figure to separate folder

    Returns
    ------
    data : dict
        Data container containing new load profiles
    """
    tech_lp = {}

    # Boiler load profile from Robert Sansom
    tech_lp['rs_lp_heating_boilers_dh'] = read_data.read_load_shapes_tech(
        paths['path_hourly_gas_shape_resid'])

    # CHP load profile from Robert Sansom
    tech_lp['rs_lp_heating_CHP_dh'] = read_data.read_load_shapes_tech(
        paths['lp_all_microchp_dh'])

    # Heat pump load profile from Love et al. (2017)
    tech_lp['rs_lp_heating_hp_dh'] = read_data.read_load_shapes_tech(
        paths['lp_elec_hp_dh'])

    #tech_lp['rs_shapes_cooling_dh'] = read_data.read_load_shapes_tech(paths['path_shape_rs_cooling']) #Not implemented
    tech_lp['ss_shapes_cooling_dh'] = read_data.read_load_shapes_tech(paths['path_shape_ss_cooling'])

    # Add fuel data of other model enduses to the fuel data table (E.g. ICT or wastewater)
    tech_lp['rs_lp_storage_heating_dh'] = read_data.read_load_shapes_tech(
        paths['lp_elec_storage_heating'])
    tech_lp['rs_lp_second_heating_dh'] = read_data.read_load_shapes_tech(
        paths['lp_elec_secondary_heating'])

    # --------------------------------------------
    # Print individualtechnology load profiles of technologies
    # --------------------------------------------
    if plot_tech_lp:

        # Maybe move to result folder in a later step
        path_folder_lp = os.path.join(local_paths['local_path_datafolder'], 'individual_lp')
        basic_functions.create_folder(path_folder_lp)

        # Boiler
        fig_lp.plot_lp_dh(
            tech_lp['rs_lp_heating_boilers_dh']['workday'] * 100,
            path_folder_lp,
            "{}".format("heating_boilers_workday"))
        fig_lp.plot_lp_dh(
            tech_lp['rs_lp_heating_boilers_dh']['holiday'] * 100,
            path_folder_lp,
            "{}".format("heating_boilers_holiday"))
        fig_lp.plot_lp_dh(
            tech_lp['rs_lp_heating_boilers_dh']['peakday'] * 100,
            path_folder_lp,
            "{}".format("heating_boilers_peakday"))

        # CHP
        fig_lp.plot_lp_dh(
            tech_lp['rs_lp_heating_hp_dh']['workday'] * 100,
            path_folder_lp,
            "{}".format("heatpump_workday"))
        fig_lp.plot_lp_dh(
            tech_lp['rs_lp_heating_hp_dh']['holiday'] * 100,
            path_folder_lp,
            "{}".format("heatpump_holiday"))
        fig_lp.plot_lp_dh(
            tech_lp['rs_lp_heating_hp_dh']['peakday'] * 100,
            path_folder_lp,
            "{}".format("heatpump_peakday"))

        # HP
        fig_lp.plot_lp_dh(
            tech_lp['rs_lp_heating_CHP_dh']['workday'] * 100,
            path_folder_lp,
            "{}".format("heating_CHP_workday"))
        fig_lp.plot_lp_dh(
            tech_lp['rs_lp_heating_CHP_dh']['holiday'] * 100,
            path_folder_lp,
            "{}".format("heating_CHP_holiday"))
        fig_lp.plot_lp_dh(
            tech_lp['rs_lp_heating_CHP_dh']['peakday'] * 100,
            path_folder_lp,
            "{}".format("heating_CHP_peakday"))

        # Stroage heating
        fig_lp.plot_lp_dh(
            tech_lp['rs_lp_storage_heating_dh']['workday'] * 100,
            path_folder_lp,
            "{}".format("storage_heating_workday"))
        fig_lp.plot_lp_dh(
            tech_lp['rs_lp_storage_heating_dh']['holiday'] * 100,
            path_folder_lp,
            "{}".format("storage_heating_holiday"))
        fig_lp.plot_lp_dh(
            tech_lp['rs_lp_storage_heating_dh']['peakday'] * 100,
            path_folder_lp,
            "{}".format("storage_heating_peakday"))

        # Direct electric heating
        fig_lp.plot_lp_dh(
            tech_lp['rs_lp_second_heating_dh']['workday'] * 100,
            path_folder_lp,
            "{}".format("secondary_heating_workday"))
        fig_lp.plot_lp_dh(
            tech_lp['rs_lp_second_heating_dh']['holiday'] * 100,
            path_folder_lp,
            "{}".format("secondary_heating_holiday"))
        fig_lp.plot_lp_dh(
            tech_lp['rs_lp_second_heating_dh']['peakday'] * 100,
            path_folder_lp,
            "{}".format("secondary_heating_peakday"))

    return tech_lp

def load_data_profiles(
        paths,
        local_paths,
        model_yeardays,
        model_yeardays_daytype,
    ):
    """Collect load profiles from txt files

    Arguments
    ----------
    paths : dict
        Paths
    local_paths : dict
        Loal Paths
    model_yeardays : int
        Number of modelled yeardays
    model_yeardays_daytype : int
        Daytype of every modelled day
    """
    tech_lp = {}

    # ------------------------------------
    # Technology specific load profiles
    # ------------------------------------
    tech_lp = load_tech_profiles(
        tech_lp,
        paths,
        local_paths,
        plot_tech_lp=False) # Plot individual load profiles

    # Load enduse load profiles
    tech_lp['rs_shapes_dh'], tech_lp['rs_shapes_yd'] = rs_collect_shapes_from_txts(
        local_paths['rs_load_profile_txt'], model_yeardays)

    tech_lp['ss_shapes_dh'], tech_lp['ss_shapes_yd'] = ss_collect_shapes_from_txts(
        local_paths['ss_load_profile_txt'], model_yeardays)

    # -- From Carbon Trust (service sector data) read out enduse specific shapes
    tech_lp['ss_all_tech_shapes_dh'], tech_lp['ss_all_tech_shapes_yd'] = ss_read_shapes_enduse_techs(
        tech_lp['ss_shapes_dh'], tech_lp['ss_shapes_yd'])

    # ------------------------------------------------------------
    # Calculate yh load profiles for individual technologies
    # ------------------------------------------------------------

    # Heat pumps by Love
    tech_lp['rs_profile_hp_y_dh'] = get_shape_every_day(
        tech_lp['rs_lp_heating_hp_dh'], model_yeardays_daytype)

    # Storage heater
    tech_lp['rs_profile_storage_heater_y_dh'] = get_shape_every_day(
        tech_lp['rs_lp_storage_heating_dh'], model_yeardays_daytype)

    # Electric heating
    tech_lp['rs_profile_elec_heater_y_dh'] = get_shape_every_day(
        tech_lp['rs_lp_second_heating_dh'], model_yeardays_daytype)

    # Boilers
    tech_lp['rs_profile_boilers_y_dh'] = get_shape_every_day(
        tech_lp['rs_lp_heating_boilers_dh'], model_yeardays_daytype)

    # Micro CHP
    tech_lp['rs_profile_chp_y_dh'] = get_shape_every_day(
        tech_lp['rs_lp_heating_CHP_dh'], model_yeardays_daytype)

    # Service Cooling tech
    tech_lp['ss_profile_cooling_y_dh'] = get_shape_every_day(
        tech_lp['ss_shapes_cooling_dh'], model_yeardays_daytype)

    return tech_lp

def get_shape_every_day(tech_lp, model_yeardays_daytype):
    """Generate yh shape based on the daytype of
    every day in year. This function iteraes every day
    of the base year and assigns daily profiles depending
    on the daytype for every day

    Arguments
    ---------
    tech_lp : dict
        Technology load profiles
    model_yeardays_daytype : list
        List with the daytype of every modelled day

    Return
    ------
    load_profile_y_dh : dict
        Fuel profiles yh (total sum for a fully ear is 365,
        i.e. the load profile is given for every day)
    """
    # Load profiles for a single day
    lp_holiday = tech_lp['holiday'] / np.sum(tech_lp['holiday'])
    lp_workday = tech_lp['workday'] / np.sum(tech_lp['workday'])

    load_profile_y_dh = np.zeros((365, 24), dtype="float")

    for day_array_nr, day_type in enumerate(model_yeardays_daytype):
        if day_type == 'holiday':
            load_profile_y_dh[day_array_nr] = lp_holiday
        else:
            load_profile_y_dh[day_array_nr] = lp_workday

    return load_profile_y_dh

def  load_weather_stations_df(path_stations):
    """Read Weather stations from file
    """
    out_stations = {}
    stations = pd.read_csv(path_stations)

    for i in stations.index:
        station_id = stations.at[i,'station_id']
        latitude = stations.at[i,'latitude']
        longitude = stations.at[i,'longitude']

        out_stations[station_id] = {
            'latitude' : float(latitude),
            'longitude': float(longitude)}

    return out_stations

def load_weather_stations_csv(path_stations):
    """Read Weather stations from file
    """
    out_stations = {}
    stations = pd.read_csv(path_stations)

    for i in stations.index:
        station_id = stations.at[i, 'station_id']
        latitude = stations.at[i, 'latitude']
        longitude = stations.at[i, 'longitude']

        out_stations[station_id] = {
            'latitude' : float(latitude),
            'longitude': float(longitude)}

    return out_stations

def load_temp_data(
        local_paths,
        sim_yrs,
        weather_realisation,
        path_weather_data,
        same_base_year_weather=False,
        crit_temp_min_max=False,
        load_np=False,
        load_parquet=False,
        load_csv=True
    ):
    """Read in cleaned temperature and weather station data

    Arguments
    ----------
    local_paths : dict
        Local local_paths
    weather_yr_scenario : list
        Years to use temperatures
    same_base_year_weather : bool
        Criteria whether the base year weather is used for full simulation

    Returns
    -------
    weather_stations : dict
        Weather stations
    temp_data : dict
        Temperaturesv
    crit_temp_min_max : dict
        True: Hourly temperature data are provided
        False: min and max temperature are provided

    t_yrs_stations : dict
        Temperatures {sim_yr: {stations:  {t_min: np.array(365), t_max: np.array(365)}}}

    Info
    ----
    PAarquest file http://pandas.pydata.org/pandas-docs/stable/io.html#io-parquet
    """
    logging.debug("... loading temperatures", flush=True)

    temp_data_short = defaultdict(dict)
    weather_stations_with_data = defaultdict(dict)

    if crit_temp_min_max:

        # ------------------
        # Read stations
        # ------------------
        path_stations = os.path.join(path_weather_data, "stations_{}.csv".format(weather_realisation))

        weather_stations_with_data = load_weather_stations_csv(path_stations)

        # ------------------
        # Read temperatures
        # ------------------
        if load_np:
            path_temp_data = os.path.join(path_weather_data, "weather_data_{}.npy".format(weather_realisation))
            full_data = np.load(path_temp_data)

            # Convert npy to dataframe
            df_full_data = pd.DataFrame(
                full_data,
                columns=['timestep', 'station_id', 'stiching_name', 'yearday', 't_min', 't_max'])
        if load_parquet:
            path_temp_data = os.path.join(path_weather_data, "weather_data_{}.parquet".format(weather_realisation))
            df_full_data = pd.read_parquet(path_temp_data, engine='pyarrow')
        if load_csv:
            path_temp_data = os.path.join(path_weather_data, "weather_data_{}.csv".format(weather_realisation))
            df_full_data = pd.read_csv(path_temp_data)

        for sim_yr in sim_yrs:

            if same_base_year_weather:
                sim_yr = sim_yrs[0]
            else:
                pass
            print("    ... year: {}".format(sim_yr), flush=True)

            # Select all station values
            df_timestep = df_full_data.loc[df_full_data['timestep'] == sim_yr]

            for station_id in weather_stations_with_data:

                df_timestep_station = df_timestep.loc[df_timestep['station_id'] == station_id]

                # Remove extrated rows to speed up process
                df_timestep = df_timestep.drop(list(df_timestep_station.index))

                t_min = list(df_timestep_station['t_min'].values)
                t_max = list(df_timestep_station['t_max'].values)

                temp_data_short[sim_yr][station_id] = {
                    't_min': np.array(t_min),
                    't_max': np.array(t_max)}

        return dict(weather_stations_with_data), dict(temp_data_short)
    else:
        # Reading in hourly temperature data
        weather_stations = read_weather_stations_raw(
            local_paths['folder_path_weater_stations'])

        for weather_yr_scenario in sim_yrs:
            temp_data = read_weather_data.read_weather_data_script_data(
                local_paths['weather_data'], weather_yr_scenario)

            for station in weather_stations:
                try:
                    _ = temp_data[station]

                    # Remove all non-uk stations
                    if weather_stations[station]['longitude'] > 2 or weather_stations[station]['longitude'] < -8.5:
                        pass
                    else:
                        temp_data_short[weather_yr_scenario][station] = temp_data[station]
                except:
                    logging.debug("no data for weather station " + str(station))

            for station_id in temp_data_short[weather_yr_scenario].keys():
                try:
                    weather_stations_with_data[station_id] = weather_stations[station_id]
                except:
                    del temp_data_short[weather_yr_scenario][station_id]

            logging.info(
                "Info: Number of weather stations: {} weather_yr_scenario: Number of temp data: {}, weather_yr_scenario: {}".format(
                    len(weather_stations_with_data), len(temp_data_short[weather_yr_scenario]), weather_yr_scenario))

        return dict(weather_stations_with_data), dict(temp_data_short)

def load_fuels(paths):
    """Load in ECUK fuel data, enduses and sectors

    Sources:
        Residential:    Table 3.02, Table 3.08
        Service:        Table 5.5a
        Industry:       Table 4.04

    Arguments
    ---------
    paths : dict
        Paths container

    Returns
    -------
    enduses : dict
        Enduses for every submodel
    sectors : dict
        Sectors for every submodel
    fuels : dict
        yearly fuels for every submodel
    enduses_lookup : dict
        Lookup of end uses
    lookup_sector_enduses : dict
        Sector enduse lookup
    """
    submodels_names = lookup_tables.basic_lookups()['submodels_names']
    fueltypes_nr = lookup_tables.basic_lookups()['fueltypes_nr']

    enduses, sectors, fuels = {}, {}, {}
    enduses_lookup = {}

    # -------------------------------
    # submodels_names[0]: Residential SubmodelSubmodel
    # -------------------------------
    rs_fuel_raw, sectors[submodels_names[0]], enduses[submodels_names[0]] = read_data.read_fuel_rs(
        paths['rs_fuel_raw'])

    # -------------------------------
    # submodels_names[1]: Service Submodel
    # -------------------------------
    ss_fuel_raw, sectors[submodels_names[1]], enduses[submodels_names[1]] = read_data.read_fuel_ss(
        paths['ss_fuel_raw'], fueltypes_nr)

    # -------------------------------
    # submodels_names[2]: Industry
    # -------------------------------
    is_fuel_raw, sectors[submodels_names[2]], enduses[submodels_names[2]] = read_data.read_fuel_is(
        paths['is_fuel_raw'], fueltypes_nr)

    # Convert energy input units
    fuels[submodels_names[0]] = conversions.convert_fueltypes_sectors_ktoe_gwh(rs_fuel_raw)
    fuels[submodels_names[1]] = conversions.convert_fueltypes_sectors_ktoe_gwh(ss_fuel_raw)
    fuels[submodels_names[2]] = conversions.convert_fueltypes_sectors_ktoe_gwh(is_fuel_raw)

    # Aggregate fuel across sectors
    fuels['aggr_sector_fuels'] = {}
    for submodel in enduses:
        for enduse in enduses[submodel]:
            fuels['aggr_sector_fuels'][enduse] = sum(fuels[submodel][enduse].values())

    cnt = 0
    for submodel in enduses:
        for enduse in enduses[submodel]:
            enduses_lookup[enduse] = cnt
            cnt += 1

    lookup_sector_enduses = {}
    for submodel_nr, submodel in enumerate(enduses):
        lookup_sector_enduses[submodel_nr] = []
        for enduse in enduses[submodel]:
            enduse_nr = enduses_lookup[enduse]
            lookup_sector_enduses[submodel_nr].append(enduse_nr)

    return enduses, sectors, fuels, enduses_lookup, lookup_sector_enduses

def rs_collect_shapes_from_txts(txt_path, model_yeardays):
    """All pre-processed load shapes are read in from .txt files
    without accessing raw files

    This loads HES files for residential sector

    Arguments
    ----------
    data : dict
        Data
    txt_path : float
        Path to folder with stored txt files

    Return
    ------
    rs_shapes_dh : dict
        Residential yh shapes
    rs_shapes_yd : dict
        Residential yd shapes
    """
    rs_shapes_dh = {}
    rs_shapes_yd = {}

    # Iterate folders and get all enduse
    all_csv_in_folder = os.listdir(txt_path)

    enduses = set([])
    for file_name in all_csv_in_folder:
        enduse = file_name.split("__")[0]
        enduses.add(enduse)

    # Read load shapes from txt files for enduses
    for enduse in enduses:
        shape_peak_dh = read_data.read_np_array_from_txt(
            os.path.join(txt_path, str(enduse) + str("__") + str('shape_peak_dh') + str('.txt')))
        shape_non_peak_y_dh = read_data.read_np_array_from_txt(
            os.path.join(txt_path, str(enduse) + str("__") + str('shape_non_peak_y_dh') + str('.txt')))
        shape_non_peak_yd = read_data.read_np_array_from_txt(
            os.path.join(txt_path, str(enduse) + str("__") + str('shape_non_peak_yd') + str('.txt')))

        # Select only modelled days (nr_of_days, 24)
        shape_non_peak_y_dh_selection = shape_non_peak_y_dh[model_yeardays]
        shape_non_peak_yd_selection = shape_non_peak_yd[model_yeardays]

        rs_shapes_dh[enduse] = {
            'shape_peak_dh': shape_peak_dh,
            'shape_non_peak_y_dh': shape_non_peak_y_dh_selection}

        rs_shapes_yd[enduse] = {
            'shape_non_peak_yd': shape_non_peak_yd_selection}

    return rs_shapes_dh, rs_shapes_yd


def ss_collect_shapes_from_txts(txt_path, model_yeardays):
    """Collect service shapes from txt files for every setor and enduse

    Arguments
    ----------
    txt_path : string
        Path to txt shapes files
    model_yeardays : array
        Modelled yeardays

    Return
    ------
    data : dict
        Data
    """
    # Iterate folders and get all sectors and enduse from file names
    all_csv_in_folder = os.listdir(txt_path)

    enduses = set([])
    sectors = set([])
    for file_name in all_csv_in_folder:
        sector = file_name.split("__")[0]
        enduse = file_name.split("__")[1]
        enduses.add(enduse)
        sectors.add(sector)

    ss_shapes_dh = defaultdict(dict)
    ss_shapes_yd = defaultdict(dict)

    # Read load shapes from txt files for enduses
    for enduse in enduses:
        for sector in sectors:
            joint_string_name = str(sector) + "__" + str(enduse)

            shape_peak_dh = read_data.read_np_array_from_txt(
                os.path.join(
                    txt_path,
                    str(joint_string_name) + str("__") + str('shape_peak_dh') + str('.txt')))
            shape_non_peak_y_dh = read_data.read_np_array_from_txt(
                os.path.join(
                    txt_path,
                    str(joint_string_name) + str("__") + str('shape_non_peak_y_dh') + str('.txt')))
            shape_non_peak_yd = read_data.read_np_array_from_txt(
                os.path.join(
                    txt_path,
                    str(joint_string_name) + str("__") + str('shape_non_peak_yd') + str('.txt')))

            # -----------------------------------------------------------
            # Select only modelled days (nr_of_days, 24)
            # -----------------------------------------------------------
            shape_non_peak_y_dh_selection = shape_non_peak_y_dh[model_yeardays]
            shape_non_peak_yd_selection = shape_non_peak_yd[model_yeardays]

            ss_shapes_dh[enduse][sector] = {
                'shape_peak_dh': shape_peak_dh,
                'shape_non_peak_y_dh': shape_non_peak_y_dh_selection}

            ss_shapes_yd[enduse][sector] = {
                'shape_non_peak_yd': shape_non_peak_yd_selection}

    return dict(ss_shapes_dh), dict(ss_shapes_yd)

def create_enduse_dict(data, rs_fuel_raw_data_enduses):
    """Create dictionary with all residential enduses and store in data dict

    For residential model

    Arguments
    ----------
    data : dict
        Main data dictionary

    rs_fuel_raw_data_enduses : dict
        Raw fuel data from external enduses (e.g. other models)

    Returns
    -------
    enduses : list
        Ditionary with residential enduses
    """
    enduses = []
    for ext_enduse in data['external_enduses_resid']:
        enduses.append(ext_enduse)

    for enduse in rs_fuel_raw_data_enduses:
        enduses.append(enduse)

    return enduses

def ss_read_shapes_enduse_techs(ss_shapes_dh, ss_shapes_yd):
    """Iterate carbon trust dataset and read out shapes for enduses.

    Arguments
    ----------
    ss_shapes_yd : dict
        Data

    Returns
    -------

    Read out enduse shapes to assign fuel shape for specific technologies
    in service sector. Because no specific shape is provided for service sector,
    the overall enduse shape is used for all technologies

    Note
    ----
    The first setor is selected and all shapes of the enduses of this
    sector read out. Because all enduses exist for each sector,
    it does not matter from which sector the shapes are talen from
    """
    ss_all_tech_shapes_dh = {}
    ss_all_tech_shapes_yd = {}

    for enduse in ss_shapes_yd:
        for sector in ss_shapes_yd[enduse]:
            ss_all_tech_shapes_dh[enduse] = ss_shapes_dh[enduse][sector]
            ss_all_tech_shapes_yd[enduse] = ss_shapes_yd[enduse][sector]
            break #only iterate first sector as all enduses are the same in all sectors

    return ss_all_tech_shapes_dh, ss_all_tech_shapes_yd

def read_scenario_data(path_to_csv, value_name='value', region_name='region'):
    """
    """
    data = {}

    if not os.path.exists(path_to_csv):
        raise FileNotFoundError("Cannot find file {}".format(path_to_csv))

    with open(path_to_csv, 'r') as csvfile:
        rows = csv.reader(csvfile, delimiter=',')
        headings = next(rows)  # Skip first row
        for row in rows:

            region = str(row[read_data.get_position(headings, region_name)])
            year = int(float(row[read_data.get_position(headings, 'timestep')]))
            value = float(row[read_data.get_position(headings, value_name)])

            try:
                data[year][region] = value
            except KeyError:
                data[year] = {}
                data[year][region] = value

    return data

def read_scenario_data_gva(path_to_csv, region_name='region', value_name='value', all_dummy_data=False):
    """Function to read in GVA locally

    IF no value, provide with dummy value "1"

    if all_dummy_data == True, then all is dummy data and
    constant over time
    """
    out_dict = {}

    with open(path_to_csv, 'r') as csvfile:
        rows = csv.reader(csvfile, delimiter=',')
        headings = next(rows)
        for row in rows:

            # --------------
            # All dummy data
            # --------------
            if all_dummy_data:
                region = str(row[read_data.get_position(headings, region_name)])
                for year_dummy in range(2015, 2051):
                    for sector_dummy in range(1, 47):
                        dummy_sector_value = 1

                        try:
                            out_dict[year_dummy][region][sector_dummy] = dummy_sector_value
                        except KeyError:
                            out_dict[year_dummy] = defaultdict(dict)
                            out_dict[year_dummy][region][sector_dummy] = dummy_sector_value

            else:
                if row[read_data.get_position(headings, 'timestep')] == '': #No data provided
                    region = str(row[read_data.get_position(headings, region_name)])
                    for year_dummy in range(2015, 2051):
                        for sector_dummy in range(1, 47):
                            dummy_sector_value = 1
                            out_dict[year_dummy][region][sector_dummy] = dummy_sector_value
                else:
                    region = str(row[read_data.get_position(headings, region_name)])
                    year = int(float(row[read_data.get_position(headings, 'timestep')]))
                    value = float(row[read_data.get_position(headings, value_name)])
                    economic_sector__gor = float(row[read_data.get_position(headings, 'economic_sector__gor')])
                try:
                    out_dict[year][region][economic_sector__gor] = value
                except KeyError:
                    out_dict[year] = defaultdict(dict)
                    out_dict[year][region][economic_sector__gor] = value

    # Convert to regular dict
    for key, value in out_dict.items():
        out_dict[key] = dict(value)

    return out_dict

def read_employment_stats(path_to_csv):
    """Read in employment statistics per LAD.

    This dataset provides 2011 estimates that classify usual
    residents aged 16 to 74 in employment the week before
    the census in United Kingdom by industry.

    Outputs
    -------
    data : dict
        geocode, Nr_of_category

    Infos
    ------
    Industry: All categories: Industry
    Industry: A Agriculture, forestry and fishing
    Industry: B Mining and quarrying
    Industry: C Manufacturing
    Industry: C10-12 Manufacturing: Food, beverages and tobacco
    Industry: C13-15 Manufacturing: Textiles, wearing apparel and leather and related products
    Industry: C16,17 Manufacturing: Wood, paper and paper products
    Industry: C19-22 Manufacturing: Chemicals, chemical products, rubber and plastic
    Industry: C23-25 Manufacturing: Low tech
    Industry: C26-30 Manufacturing: High tech
    Industry: C18, 31, 32 Manufacturing: Other
    Industry: D Electricity, gas, steam and air conditioning supply
    Industry: E Water supply, sewerage, waste management and remediation activities
    Industry: F Construction
    Industry: G Wholesale and retail trade; repair of motor vehicles and motor cycles
    Industry: H Transport and storage
    Industry: I Accommodation and food service activities
    Industry: J Information and communication
    Industry: K Financial and insurance activities
    Industry: L Real estate activities
    Industry: M Professional, scientific and technical activities
    Industry: N Administrative and support service activities
    Industry: O Public administration and defence; compulsory social security
    Industry: P Education
    Industry: Q Human health and social work activities
    Industry: R,S Arts, entertainment and recreation; other service activities
    Industry: T Activities of households as employers; undifferentiated goods - and services - producing activities of households for own use
    Industry: U Activities of extraterritorial organisations and bodies
    """
    data = defaultdict(dict)

    with open(path_to_csv, 'r') as csvfile:
        lines = csv.reader(csvfile, delimiter=',')
        headings = next(lines) # Skip first row

        for line in lines:
            geocode = str.strip(line[2])

            # Iterate fields and copy values
            for counter, heading in enumerate(headings[4:], 4):
                heading_split = heading.split(":")
                category_unclean = str.strip(heading_split[1])
                category_full_name = str.strip(category_unclean.split(" ")[0]).replace(" ", "_")
                category_nr = category_full_name.split("_")[0]

                data[geocode][category_nr] = float(line[counter])

    logging.info("... loaded employment statistics")
    return dict(data)



"""Script to prepare weather data
Load weather simulation data

1. Download yearly data from here: http://catalogue.ceda.ac.uk/uuid/0cea8d7aca57427fae92241348ae9b03

2. Extract data

3. Run this script to get only the relevant data

Links
------
http://data.ceda.ac.uk//badc//weather_at_home/data/marius_time_series/CEDA_MaRIUS_Climate_Data_Description.pdf
http://data.ceda.ac.uk/badc/weather_at_home/data/marius_time_series/near_future/data/


https://medium.com/@rtjeannier/pandas-101-cont-9d061cb73bfc

"""
import logging
import os
import pytemperature
import xarray as xr
import numpy as np
import pandas as pd

from energy_demand.basic import basic_functions
from energy_demand.read_write import write_data

def create_realisation(
        base_yr_remapped_weather_path,
        realisation_list,
        realisation_path,
        realisation_out_path,
        path_stiching_table
    ):
    """
    Before running, generate 2015 remapped data
    """
    sim_yr_start = 2015
    sim_yr_end = 2050 + 1

    print("... writing data", flush=True)
    write_to_csv = True
    write_to_np = False
    write_to_parquet = False

    # Create result path
    basic_functions.create_folder(realisation_out_path)

    # Read in stiching table
    df_path_stiching_table = pd.read_table(path_stiching_table, sep=" ")

    # Set year as index
    df_path_stiching_table = df_path_stiching_table.set_index('year')

    # Realisations
    realisations = list(df_path_stiching_table.columns)

    columns = ['timestep', 'longitude', 'latitude', 'yearday', 'wss', 'rlds', 'rsds']

    for i in realisation_list:
        realisation = realisations[i]

        print("... creating weather data for realisation " + str(realisation), flush=True)
        realisation_out = []

        for sim_yr in range(sim_yr_start, sim_yr_end):

            print("   ... year: " + str(sim_yr), flush=True)
            # If year 2015 - 2019, take base year weather
            if sim_yr in range(2015, 2020):
                #print("... for year '{}' data from the year 2015 are used".format(sim_yr))
                year = 2020 #Select year 2020
                stiching_name = df_path_stiching_table[realisation][year]
                path_weather_data = os.path.join(realisation_path, str(year), stiching_name)
                path_wss = os.path.join(path_weather_data, "wss.npy")
                path_rlds = os.path.join(path_weather_data, "rlds.npy")
                path_rsds = os.path.join(path_weather_data, "rsds.npy")
                path_stations = os.path.join(path_weather_data, "stations.csv")

            elif sim_yr == 2050:
                #print("... for year '{}' data from the year 2049 are used".format(sim_yr))
                year = 2049
                stiching_name = df_path_stiching_table[realisation][year]
                path_weather_data = os.path.join(realisation_path, str(year), stiching_name)
                path_wss = os.path.join(path_weather_data, "wss.npy")
                path_rlds = os.path.join(path_weather_data, "rlds.npy")
                path_rsds = os.path.join(path_weather_data, "rsds.npy")
                path_stations = os.path.join(path_weather_data, "stations.csv")
            else:
                #print("normal")
                year = sim_yr
                stiching_name = df_path_stiching_table[realisation][year]
                path_weather_data = os.path.join(realisation_path, str(year), stiching_name)
                path_wss = os.path.join(path_weather_data, "wss.npy")
                path_rlds = os.path.join(path_weather_data, "rlds.npy")
                path_rsds = os.path.join(path_weather_data, "rsds.npy")
                path_stations = os.path.join(path_weather_data, "stations.csv")

            # Read t_min, t_max, stations)
            wss = np.load(path_wss)
            rlds = np.load(path_rlds)
            rsds = np.load(path_rsds)
            stations = pd.read_csv(path_stations)
            stations['timestep'] = sim_yr

            nr_stations_array = len(list(stations.values))

            for station_cnt in range(nr_stations_array):
                wss_station = wss[station_cnt]
                rlds_station = rlds[station_cnt]
                rsds_station = rsds[station_cnt]
                station_long = stations.loc[station_cnt]['longitude']
                station_lat = stations.loc[station_cnt]['latitude']
                for yearday in range(365):
                    realisation_out.append(
                        [sim_yr, station_long, station_lat, yearday, wss_station[yearday], rlds_station[yearday], rsds_station[yearday]])

        print("...writing out")
        # Write data to csv
        if write_to_csv:
            df = pd.DataFrame(realisation_out, columns=columns)
            path_out_csv = os.path.join(realisation_out_path, "weather_data_{}.csv".format(realisation))
            df.to_csv(path_out_csv, index=False)

        '''if write_to_parquet: 
            path_out_parquet = os.path.join(realisation_out_path, "weather_data_{}.parquet".format(realisation))
            df = pd.DataFrame(realisation_out, columns=columns)
            df.to_parquet(path_out_parquet, engine='pyarrow')

        if write_to_np:
            path_out_array = os.path.join(realisation_out_path, "weather_data_{}.npy".format(realisation))
            out_array = np.array(realisation_out)
            np.save(path_out_array, out_array)'''

        # Write stations to csv
        print("... writing stations to csv", flush=True)
        #stations_out.to_csv(os.path.join(realisation_out_path, "stations_{}.csv".format(realisation)), index=False)

def get_temp_data_from_nc(path_nc_file, attribute_to_keep):
    """Open c file, convert it into dataframe,
    clean it and extract attribute data
    """
    # Open as xarray
    x_array_data = xr.open_dataset(path_nc_file)

    # Convert to dataframe
    df = x_array_data.to_dataframe()

    # Add index as columns
    df = df.reset_index()

    # Drop unnecessary columns
    df = df.drop(['time_bnds', 'rotated_latitude_longitude', 'rlat', 'rlon', 'bnds', 'height'], axis=1)

    # Drop all missing value
    df = df.dropna(subset=[attribute_to_keep])

    return df

def convert_to_celcius(df, attribute_to_convert):
    """# Convert Kelvin to Celsius (# Kelvin to Celsius)
    """
    df[attribute_to_convert] = df[attribute_to_convert].apply(pytemperature.k2c)
    return df

def extend_360_day_to_365(df, attribute):
    """Add evenly distributed days across the 360 days

    Return
    ------
    out_list : list
        Returns list with days with the following attributes
        e.g. [[lat, long, yearday365, value]]
    """
    # Copy the following position of day in the 360 year
    days_to_duplicate = [60, 120, 180, 240, 300]
    out_list = []

    # Copy every 60th day over the year and duplicate it. Do this five times --> get from 360 to 365 days
    day_cnt, day_cnt_365 = 0, 0
    for i in df.index:
        day_cnt += 1

        latitude = df.get_value(i, 'lat')
        longitude = df.get_value(i, 'lon')
        value = df.get_value(i, attribute)

        # Copy extra day
        if day_cnt in days_to_duplicate:
            out_list.append([latitude, longitude, day_cnt_365, value])
            day_cnt_365 += 1

        # Copy day
        day_cnt_365 += 1
        out_list.append([latitude, longitude, day_cnt_365, value])

        # Reset counter if next weather station
        if day_cnt == 360:
            day_cnt = 0
            day_cnt_365 = 0

    return out_list

def write_weather_data(data_list):
    """Write weather data to array
    data_list : list
        [[lat, long, yearday365, value]]
    """
    station_coordinates = []

    assert not len(data_list) % 365 #Check if dividable by 365
    nr_stations = int(len(data_list) / 365)

    stations_data = np.zeros((nr_stations, 365))
    station_data = np.zeros((365))

    station_id_cnt = 0
    cnt = 0

    for row in data_list:
        station_data[cnt] = row[3]

        if cnt == 364:

            # 365 day data for weather station
            stations_data[station_id_cnt] = station_data

            # Weather station metadata
            station_lon = row[1]
            station_lat = row[0]

            station_id = "station_id_{}".format(station_id_cnt)

            #ID, latitude, longitude
            station_coordinates.append([station_id, station_lat, station_lon])

            # Reset
            station_data = np.zeros((365))
            station_id_cnt += 1
            cnt = -1
        cnt += 1

    return station_coordinates, stations_data

def weather_dat_prepare(data_path, result_path, years_to_clean=range(2020, 2049)):
    """
    """
    print("folder_name " + str(years_to_clean))

    # Create reulst folders
    result_folder = os.path.join(result_path, "_weather_data_cleaned_modassar")
    basic_functions.create_folder(result_folder)

    for folder_name in years_to_clean:

        year = folder_name

        # Create folder
        path_year = os.path.join(result_folder, str(year))
        basic_functions.create_folder(path_year)

        path_realizations = os.path.join(path, str(year))
        realization_names = os.listdir(path_realizations)

        for realization_name in realization_names:
            print("... processing {}  {}".format(str(year), str(realization_name)), flush=True)

            # Create folder
            path_realization = os.path.join(path_year, realization_name)
            basic_functions.create_folder(path_realization)
            
            # Data to extract
            path_wind = os.path.join(path_realizations, realization_name, 'daily', 'WAH_{}_wss_daily_g2_{}.nc'.format(realization_name, year))
            path_rlds = os.path.join(path_realizations, realization_name, 'daily', 'WAH_{}_rlds_daily_g2_{}.nc'.format(realization_name, year))
            path_rsds = os.path.join(path_realizations, realization_name, 'daily', 'WAH_{}_rsds_daily_g2_{}.nc'.format(realization_name, year))
            
            # Load data
            print("     ..load data", flush=True)
            wss = get_temp_data_from_nc(path_wind, 'wss')
            rlds = get_temp_data_from_nc(path_rlds, 'rlds')
            rsds = get_temp_data_from_nc(path_rsds, 'rsds')

            # Convert 360 day to 365 days
            print("     ..extend day", flush=True)
            list_wss = extend_360_day_to_365(wss, 'wss')
            list_rlds = extend_360_day_to_365(rlds, 'rlds')
            list_rsds = extend_360_day_to_365(rsds, 'rsds')

            # Write out single weather stations as numpy array
            print("     ..write out", flush=True)
            station_coordinates, stations_wss = write_weather_data(list_wss)
            station_coordinates, stations_rlds = write_weather_data(list_rlds)
            station_coordinates, stations_rsds = write_weather_data(list_rsds)

            # Write to csv
            np.save(os.path.join(path_realization, "wss.npy"), stations_wss)
            np.save(os.path.join(path_realization, "rlds.npy"), stations_rlds)
            np.save(os.path.join(path_realization, "rsds.npy"), stations_rsds)

            #write_data.write_yaml(station_coordinates, os.path.join(path_realization, "stations.yml"))
            df = pd.DataFrame(station_coordinates, columns=['station_id', 'latitude', 'longitude'])
            df.to_csv(os.path.join(path_realization, "stations.csv"), index=False)

    print("... finished cleaning weather data")

clean_original_files = False
stich_weather_scenario = True

if clean_original_files:
    # ------------------------
    # Clean scenario data
    # ------------------------
    path = "X:/nismod/data/energy_demand/J-MARIUS_data"
    result_path = "X:/nismod/data/energy_demand/J-MARIUS_data"
    
    #years_to_clean = range(2020, 2026, 1)
    #years_to_clean = range(2026, 2031, 1)
    #years_to_clean = range(2031, 2041, 1)
    years_to_clean = range(2041, 2051, 1)

    weather_dat_prepare(path, result_path, years_to_clean)

if stich_weather_scenario:
    # ------------------------
    # Create realisation
    # ------------------------
    base_yr_remapped_weather_path = "X:/nismod/data/energy_demand/J-MARIUS_data/_weather_data_cleaned_modassar/2020"
    path = "X:/nismod/data/energy_demand/J-MARIUS_data"
    result_path = "X:/nismod/data/energy_demand/J-MARIUS_data"
    realisation_path = "X:/nismod/data/energy_demand/J-MARIUS_data/_weather_data_cleaned_modassar"
    realisation_out_path = "X:/nismod/data/energy_demand/J-MARIUS_data/_weather_realisation_modassar"
    path_stiching_table = "X:/nismod/data/energy_demand/J-MARIUS_data/stitching_table/stitching_table_nf.dat"

    realisation_list = range(100)
    #realisation_list = range(19, 100)

    create_realisation(
        base_yr_remapped_weather_path=base_yr_remapped_weather_path,
        realisation_list=realisation_list,
        realisation_path=realisation_path,
        realisation_out_path=realisation_out_path,
        path_stiching_table=path_stiching_table)



"""Conversion of units
"""
from collections import defaultdict
import numpy as np

def gwh_to_twh(gwh):
    """Convert GWh to TWh

    Arguments
    ---------
    gwh : float
        GWh

    Returns
    -------
    twh : str
        TWh
    """
    twh = gwh / 1000.0
    return twh

def ktoe_to_gwh(ktoe):
    """Conversion of ktoe to gwh. As ECUK input
    ktoe per year are provided, which are converted
    into GWh per year.

    Arguments
    ----------
    ktoe : float
        Energy demand in ktoe

    Returns
    -------
    gwh : float
        Energy demand in GWh

    Notes
    -----
    https://www.iea.org/statistics/resources/unitconverter/
    """
    gwh = ktoe * 11.6300000

    return gwh

def gwh_to_ktoe(gwh):
    """Conversion of gwh to ktoe

    Arguments
    ----------
    ktoe : float
        Energy demand in ktoe

    Returns
    -------
    data_gwh : float
        Energy demand in GWh

    Notes
    -----
    https://www.iea.org/statistics/resources/unitconverter/
    """
    ktoe = gwh / 11.6300000

    return ktoe

def kwh_to_gwh(kwh):
    """"Conversion of MW to GWh

    Arguments
    ---------
    kwh : float
        Kilowatthours

    Return
    ------
    gwh : float
        Gigawatthours
    """
    gwh = kwh * 0.000001

    return gwh

def gwh_to_kwh(gwh):
    """"Conversion of MW to GWh

    Arguments
    ---------
    kwh : float
        Kilowatthours

    Return
    ------
    gwh : float
        Gigawatthours
    """
    kwh = gwh *  1000000

    return kwh

def mw_to_gwh(megawatt, number_of_hours):
    """"Conversion of MW to GWh

    Arguments
    ---------
    kwh : float
        Kilowatthours
    number_of_hours : float
        Number of hours

    Return
    ------
    gwh : float
        Gigawatthours

    """
    # Convert MW to MWh
    megawatt_hour = megawatt * number_of_hours

    # Convert mwth to gwh
    gigawatthour = megawatt_hour / 1000.0

    return gigawatthour

def ktoe_to_twh(ktoe):
    """Conversion of ktoe to TWh

    Arguments
    ----------
    ktoe : float
        Energy demand in ktoe

    Returns
    -------
    data_gwh : float
        Energy demand in TWh

    Notes
    -----
    https://www.iea.org/statistics/resources/unitconverter/
    """
    data_twh = ktoe * 0.01163

    return data_twh


def convert_fueltypes_sectors_ktoe_gwh(fuel_dict):
    """Iterate fueltypes and convert ktoe to gwh

    Arguments
    ----------
    fuel_dict : array
        Fuel per fueltype

    Returns
    -------
    fuel_converted : dict
        Array with converted fuel per fueltype
    """
    fuel_converted = defaultdict(dict)

    for enduse in fuel_dict:
        for sector, fuels in fuel_dict[enduse].items():
            fuel_converted[enduse][sector] = np.apply_along_axis(
                func1d=ktoe_to_gwh, axis=0, arr=fuels)

    return dict(fuel_converted)



"""Fig 2 figure
"""
import numpy as np
import matplotlib.pyplot as plt
#from scipy.stats import mstats
import pandas as pd
import geopandas as gpd
from scipy import stats
from shapely.geometry import Point
import matplotlib.pyplot as plt
from collections import defaultdict
from matplotlib.colors import Normalize

from energy_demand.plotting import result_mapping
from energy_demand.technologies import tech_related
from energy_demand.plotting import basic_plot_functions

def run(
        data_input,
        regions,
        _to_plot,
        fueltype_str,
        path_shapefile,
        fig_name
    ):
    """
    """
    fueltype_int = tech_related.get_fueltype_int(fueltype_str)

    # Figure related
    fig = plt.figure() #(figsize = cm2inch(10,10))
    ax = fig.add_subplot(111)
    period_h = range(8760)

    for sim_yr_to_plot in simulation_yrs_to_plot:

        # ---Collect data for every weather year
        weather_yrs_data_regional = defaultdict(dict)
        weather_yrs_data_total = []
        for weather_yr, data_weather_yr in data_input.items():

            # Weather year specific data for every region
            regions_fuel = data_weather_yr[sim_yr_to_plot][fueltype_int]

            # Total fuel
            national_fuel = np.sum(regions_fuel, axis=0)

            for region_nr, region_name in enumerate(regions):
                try:
                    weather_yrs_data_regional[region_name].append(regions_fuel[region_nr])
                except (KeyError, AttributeError):
                    weather_yrs_data_regional[region_name] = [regions_fuel[region_nr]]
            try:
                weather_yrs_data_total.append(national_fuel)
            except:
                weather_yrs_data_total = [national_fuel]

        # ---Collect data for every weather year

        # Convert regional data to dataframe
        national_fuel_array = np.array(weather_yrs_data_total)

        df = pd.DataFrame(
            national_fuel_array,
            columns=range(8760))

        # Calculate regional statistics
        mean_data = df.mean(axis=0)

        mean_data = mean_data.sort_values(ascending=False)

        # Reorder df according to ordering of mean values
        ordering_of_hours_index = list(mean_data.index)

        df = df[ordering_of_hours_index]

        #std_dev = df.std(axis=0) #standard deviation across every hour
        # Calculate quantiles
        quantile_95 = 0.95
        quantile_05 = 0.05

        df_q_95 = df.quantile(quantile_95)
        df_q_05 = df.quantile(quantile_05)

        #Transpose for plotting purposes
        df = df.T
        df_q_95 = df_q_95.T
        df_q_05 = df_q_05.T

        # ---------------
        # Smoothing lines
        # ---------------
        try:
            period_h_smoothed, df_q_95_smoothed = basic_plot_functions.smooth_data(period_h, df_q_95, num=40000)
            period_h_smoothed, df_q_05_smoothed = basic_plot_functions.smooth_data(period_h, df_q_05, num=40000)
            period_h_smoothed, mean_data_smoothed = basic_plot_functions.smooth_data(period_h, mean_data, num=40000)
        except:
            period_h_smoothed = period_h
            df_q_95_smoothed = df_q_95
            df_q_05_smoothed = df_q_05
            mean_data_smoothed = mean_data

        plt.plot(period_h_smoothed, mean_data_smoothed, color='tomato', linestyle='-', linewidth=2, label="average")
        #plt.plot(period_h_smoothed, df_q_05_smoothed, color='black', linestyle='--', linewidth=0.5, label="0.05")
        #plt.plot(period_h_smoothed, df_q_95_smoothed, color='black', linestyle='--', linewidth=0.5, label="0.95")

        # -----------------
        # Uncertainty range
        # -----------------
        plt.fill_between(
            period_h_smoothed, #x
            df_q_95_smoothed,  #y1
            df_q_05_smoothed,  #y2
            alpha=.40,
            facecolor="grey",
            label="uncertainty band")

        plt.legend(
            prop={
                'family':'arial',
                'size': 10},
            loc='best',
            frameon=False,
            shadow=True)

        plt.xlabel("Load duration curve")
        plt.ylabel("energy demand")

        plt.xlim(0, 8760)
        plt.show()

        print("--")



import numpy as np
from energy_demand.basic import conversions

def test_conversion_ktoe_gwh():
    """Testing function
    """
    in_value = 10
    expected = in_value * 11.6300000

    # call function
    out_value = conversions.ktoe_to_gwh(in_value)

    assert out_value == expected

def test_kwh_to_gwh():
    """Testing function
    """
    in_value = 10
    expected = in_value *  0.000001

    # call function
    out_value = conversions.kwh_to_gwh(in_value)

    assert out_value == expected

def test_ktoe_to_twh():
    """Testing function
    """
    in_value = 10
    expected = in_value *  0.01163

    # call function
    out_value = conversions.ktoe_to_twh(in_value)

    assert out_value == expected

def test_mw_to_gwhh():
    """Testing function
    """
    megawatt_hour = 100
    number_of_hours = 1

    expected = (megawatt_hour * number_of_hours) / 1000.0

    # call function
    out_value = conversions.mw_to_gwh(megawatt_hour, number_of_hours)

    assert out_value == expected

def test_convert_fueltypes_sectors_ktoe_gwh():
    """Testing function
    """
    in_value = {'enduse': {'sector': np.zeros((2))}}
    in_value['enduse']['sector'][0] = 10
    in_value['enduse']['sector'][1] = 20

    expected = {'enduse': {'sector': np.zeros((2))}}
    expected['enduse']['sector'][0] = 10 * 11.6300000
    expected['enduse']['sector'][1] = 20 * 11.6300000

    # call function
    out_value = conversions.convert_fueltypes_sectors_ktoe_gwh(in_value)

    np.testing.assert_array_almost_equal(out_value['enduse']['sector'][0], expected['enduse']['sector'][0])
    np.testing.assert_array_almost_equal(out_value['enduse']['sector'][1], expected['enduse']['sector'][1])



"""
"""
import os
import matplotlib.pyplot as plt

def plot_lp_dh(data_dh_modelled, path_plot_fig, fig_name):
    """plot daily profile
    """
    x_values = range(24)

    plt.plot(x_values, list(data_dh_modelled), color='red', label='modelled') #'ro', markersize=1,

    path_fig_name = os.path.join(
        path_plot_fig,
        fig_name)

    # -----------------
    # Axis
    # -----------------
    plt.ylim(0, 30)

    # ------------
    # Plot legend
    # ------------
    plt.legend(ncol=2, loc=2, frameon=False)

    # Tight layout
    plt.tight_layout()
    plt.margins(x=0)

    # Save fig
    plt.savefig(path_fig_name)
    plt.close()

'''def plot_lp_yh(data_dh_modelled, plotshow=False):
    """plot yearly profile
    """
    x_values = range(8760)

    yh_data_dh_modelled = np.reshape(data_dh_modelled, 8760)
    plt.plot(x_values, list(yh_data_dh_modelled), color='red', label='modelled') #'ro', markersize=1,

    # -----------------
    # Axis
    # -----------------
    #plt.ylim(0, 30)

    # ------------
    # Plot legend
    # ------------
    plt.legend(ncol=2, loc=2, frameon=False)

    # Tight layout
    plt.tight_layout()
    plt.margins(x=0)

    if plotshow:
        plt.show()
        plt.close()
    else:
        plt.close()'''

'''def plot_lp_yd(data_dh_modelled, plotshow=False):
    """plot yearly profile
    """
    def close_event():
        """Timer to close window automatically
        """
        plt.close()

    fig = plt.figure()
    x_values = range(365)

    plt.plot(x_values, data_dh_modelled, color='blue', label='modelled') #'ro', markersize=1,

    # -----------
    # Plot legend
    # ------------
    plt.legend(ncol=2, loc=2, frameon=False)

    # Tight layout
    plt.tight_layout()
    plt.margins(x=0)

    if plotshow:
        plt.show()
        plt.close()
    else:
        plt.close()

    #creating a timer object and setting an interval
    timer = fig.canvas.new_timer(interval = 1500)
    timer.add_callback(close_event)'''


"""
"""
import logging
import numpy as np
from collections import defaultdict
import operator
import matplotlib.pyplot as plt

from energy_demand import enduse_func
from energy_demand.plotting import basic_plot_functions

def lad_comparison_peak(
        base_yr,
        comparison_year,
        regions,
        ed_year_fueltype_regs_yh,
        fueltype_int,
        fueltype_str,
        fig_name,
        label_points=False,
        plotshow=False
    ):
    """Compare energy demand for regions for the base yeard and
    a future year

    Arguments
    ----------
    comparison_year : int
        Year to compare base year values to

    Note
    -----
    SOURCE OF LADS:
        - Data for northern ireland is not included in that, however in BEIS dataset!
    """
    result_dict = defaultdict(dict)

    # -------------------------------------------
    # Get base year modelled demand
    # -------------------------------------------
    for year, fuels in ed_year_fueltype_regs_yh.items():
        if year == base_yr:
            for region_array_nr, reg_geocode in enumerate(regions):
                _, gw_peak_fueltyp_h = enduse_func.get_peak_day_single_fueltype(fuels[fueltype_int][region_array_nr])
                result_dict['demand_by'][reg_geocode] = gw_peak_fueltyp_h

        elif year == comparison_year:
            for region_array_nr, reg_geocode in enumerate(regions):
                _, gw_peak_fueltyp_h = enduse_func.get_peak_day_single_fueltype(fuels[fueltype_int][region_array_nr])
                result_dict['demand_cy'][reg_geocode] = gw_peak_fueltyp_h
        else:
            pass

    # -----------------
    # Sort results according to size
    # -----------------
    sorted_dict_real = sorted(
        result_dict['demand_by'].items(),
        key=operator.itemgetter(1))

    # -------------------------------------
    # Plot
    # -------------------------------------
    fig = plt.figure(
        figsize=basic_plot_functions.cm2inch(9, 8)) #width, height

    ax = fig.add_subplot(1, 1, 1)

    x_values = np.arange(0, len(sorted_dict_real), 1)

    y_real_elec_demand = []
    y_modelled_elec_demand = []

    labels = []
    for sorted_region in sorted_dict_real:
        geocode_lad = sorted_region[0]

        y_real_elec_demand.append(
            result_dict['demand_by'][geocode_lad])
        y_modelled_elec_demand.append(
            result_dict['demand_cy'][geocode_lad])

        logging.debug(
            "validation for LAD region: %s %s diff: %s",
            result_dict['demand_by'][geocode_lad],
            result_dict['demand_cy'][geocode_lad],
            result_dict['demand_cy'][geocode_lad] - result_dict['demand_by'][geocode_lad])

        labels.append(geocode_lad)

    # --------
    # Axis
    # --------
    plt.tick_params(
        axis='x',          # changes apply to the x-axis
        which='both',      # both major and minor ticks are affected
        bottom='off',      # ticks along the bottom edge are off
        top='off',         # ticks along the top edge are off
        labelbottom='off') # labels along the bottom edge are off

    # ----------------------------------------------
    # Plot
    # ----------------------------------------------
    plt.plot(
        x_values,
        y_real_elec_demand,
        linestyle='None',
        marker='o',
        markersize=1.6, #1.6
        fillstyle='full',
        markerfacecolor='grey',
        markeredgewidth=0.2,
        color='black',
        label='base year: {}'.format(base_yr))

    plt.plot(
        x_values,
        y_modelled_elec_demand,
        marker='o',
        linestyle='None',
        markersize=1.6,
        markerfacecolor='white',
        fillstyle='none',
        markeredgewidth=0.5,
        markeredgecolor='blue',
        color='black',
        label='current year: {}'.format(comparison_year))

    # Limit
    plt.ylim(ymin=0, ymax=1.2)
    # -----------
    # Labelling
    # -----------
    if label_points:
        for pos, txt in enumerate(labels):
            ax.text(
                x_values[pos],
                y_modelled_elec_demand[pos],
                txt,
                horizontalalignment="right",
                verticalalignment="top",
                fontsize=3)

    plt.xlabel("UK regions (excluding northern ireland)")
    plt.ylabel("peak h {} [GWh]".format(fueltype_str))

    # --------
    # Legend
    # --------
    plt.legend(
        prop={
            'family': 'arial',
            'size': 8},
        frameon=False)

    # Tight layout
    plt.margins(x=0)
    plt.tight_layout()
    plt.savefig(fig_name)

    if plotshow:
        plt.show()
        plt.close()
    else:
        plt.close()

def plot_lad_comparison(
        base_yr,
        comparison_year,
        regions,
        ed_year_fueltype_regs_yh,
        fueltype_int,
        fueltype_str,
        fig_name,
        label_points=False,
        plotshow=False
    ):
    """Compare energy demand for regions for the base yeard and
    a future year

    Arguments
    ----------
    comparison_year : int
        Year to compare base year values to

    Note
    -----
    SOURCE OF LADS:
        - Data for northern ireland is not included in that, however in BEIS dataset!
    """
    result_dict = defaultdict(dict)

    # -------------------------------------------
    # Get base year modelled demand
    # -------------------------------------------
    for year, fuels in ed_year_fueltype_regs_yh.items():

        if year == base_yr:
            for region_array_nr, reg_geocode in enumerate(regions):
                gw_per_region_modelled = np.sum(fuels[fueltype_int][region_array_nr])
                result_dict['demand_by'][reg_geocode] = gw_per_region_modelled
        elif year == comparison_year:
            for region_array_nr, reg_geocode in enumerate(regions):
                gw_per_region_modelled = np.sum(fuels[fueltype_int][region_array_nr])
                result_dict['demand_cy'][reg_geocode] = gw_per_region_modelled
        else:
            pass

    logging.info("Comparison: modelled: %s real: %s".format(
        sum(result_dict['demand_by'].values()),
        sum(result_dict['demand_cy'].values())))

    # -----------------
    # Sort results according to size
    # -----------------
    sorted_dict_real = sorted(
        result_dict['demand_by'].items(),
        key=operator.itemgetter(1))

    # -------------------------------------
    # Plot
    # -------------------------------------
    fig = plt.figure(
        figsize=basic_plot_functions.cm2inch(9, 8)) #width, height

    ax = fig.add_subplot(1, 1, 1)

    x_values = np.arange(0, len(sorted_dict_real), 1)

    y_real_elec_demand = []
    y_modelled_elec_demand = []

    labels = []
    for sorted_region in sorted_dict_real:
        geocode_lad = sorted_region[0]

        y_real_elec_demand.append(
            result_dict['demand_by'][geocode_lad])
        y_modelled_elec_demand.append(
            result_dict['demand_cy'][geocode_lad])

        logging.debug(
            "validation for LAD region: %s %s diff: %s",
            result_dict['demand_by'][geocode_lad],
            result_dict['demand_cy'][geocode_lad],
            result_dict['demand_cy'][geocode_lad] - result_dict['demand_by'][geocode_lad])

        labels.append(geocode_lad)

    # --------
    # Axis
    # --------
    plt.tick_params(
        axis='x',          # changes apply to the x-axis
        which='both',      # both major and minor ticks are affected
        bottom='off',      # ticks along the bottom edge are off
        top='off',         # ticks along the top edge are off
        labelbottom='off') # labels along the bottom edge are off

    # ----------------------------------------------
    # Plot
    # ----------------------------------------------
    plt.plot(
        x_values,
        y_real_elec_demand,
        linestyle='None',
        marker='o',
        markersize=1.6, #1.6
        fillstyle='full',
        markerfacecolor='grey',
        markeredgewidth=0.2,
        color='black',
        label='base year: {}'.format(base_yr))

    plt.plot(
        x_values,
        y_modelled_elec_demand,
        marker='o',
        linestyle='None',
        markersize=1.6,
        markerfacecolor='white',
        fillstyle='none',
        markeredgewidth=0.5,
        markeredgecolor='blue',
        color='black',
        label='current year: {}'.format(comparison_year))

    # Limit
    plt.ylim(ymin=0)

    # -----------
    # Labelling
    # -----------
    if label_points:
        for pos, txt in enumerate(labels):
            ax.text(
                x_values[pos],
                y_modelled_elec_demand[pos],
                txt,
                horizontalalignment="right",
                verticalalignment="top",
                fontsize=3)

    plt.xlabel("UK regions (excluding northern ireland)")
    plt.ylabel("{} [GWh]".format(fueltype_str))

    # --------
    # Legend
    # --------
    plt.legend(
        prop={
            'family': 'arial',
            'size': 8},
        frameon=False)

    # Tight layout
    plt.margins(x=0)
    plt.tight_layout()
    plt.savefig(fig_name)

    if plotshow:
        plt.show()
        plt.close()
    else:
        plt.close()


""" A ``WeatherRegion`` object is is generated per
weather station and regional load profile and technology
stocks are assigned."""
import uuid
import numpy as np

from energy_demand.technologies import technological_stock
from energy_demand.profiles import load_profile
from energy_demand.profiles import hdd_cdd
from energy_demand.basic import basic_functions
from energy_demand import enduse_func
from energy_demand.initalisations import helpers
from energy_demand.profiles import generic_shapes
from energy_demand.basic import lookup_tables

class WeatherRegion(object):
    """WeaterRegion

    Arguments
    ----------
    name : str
        Unique identifyer of weather region
    latitude, longitude : float
        Coordinates
    assumptions : dict
        Assumptions
    technologies : list
        All technology assumptions
    enduses : list
        All enduses
    temp_by, temp_ey : array
        Temperature data
    tech_lp : dict
        Technology load profiles
    sectors : list
        Sectors

    Note
    ----
    - For each region, a technology stock is defined
    - regional specific fuel shapes are assigned to technologies
    """
    def __init__(
            self,
            name,
            latitude,
            longitude,
            assumptions,
            technologies,
            enduses,
            temp_by,
            temp_cy,
            tech_lp,
            sectors,
            crit_temp_min_max
        ):
        """Constructor of weather region
        """
        self.name = name
        self.longitude = longitude
        self.latitude = latitude

        fueltypes = lookup_tables.basic_lookups()['fueltypes']

        # Base temperatures of current year
        rs_t_base_heating_cy = assumptions.non_regional_vars['rs_t_base_heating'][assumptions.curr_yr]
        ss_t_base_heating_cy = assumptions.non_regional_vars['ss_t_base_heating'][assumptions.curr_yr]
        ss_t_base_cooling_cy = assumptions.non_regional_vars['ss_t_base_cooling'][assumptions.curr_yr]
        is_t_base_heating_cy = assumptions.non_regional_vars['is_t_base_heating'][assumptions.curr_yr]

        # ==================================================================
        # Technology stocks
        # ==================================================================
        rs_tech_stock = technological_stock.TechStock(
            'rs_tech_stock',
            technologies,
            assumptions.base_yr,
            assumptions.curr_yr,
            fueltypes,
            temp_by,
            temp_cy,
            assumptions.t_bases.rs_t_heating,
            enduses['residential'],
            rs_t_base_heating_cy,
            assumptions.specified_tech_enduse_by)

        ss_tech_stock = technological_stock.TechStock(
            'ss_tech_stock',
            technologies,
            assumptions.base_yr,
            assumptions.curr_yr,
            fueltypes,
            temp_by,
            temp_cy,
            assumptions.t_bases.ss_t_heating,
            enduses['service'],
            ss_t_base_heating_cy,
            assumptions.specified_tech_enduse_by)

        is_tech_stock = technological_stock.TechStock(
            'is_tech_stock',
            technologies,
            assumptions.base_yr,
            assumptions.curr_yr,
            fueltypes,
            temp_by,
            temp_cy,
            assumptions.t_bases.is_t_heating,
            enduses['industry'],
            ss_t_base_heating_cy,
            assumptions.specified_tech_enduse_by)

        self.tech_stock = {
            'residential': rs_tech_stock,
            'service': ss_tech_stock,
            'industry': is_tech_stock}

        # ==================================================================
        # Load profiles
        # ==================================================================
        flat_shape_yd, _, flat_shape_y_dh = generic_shapes.flat_shape()

        # ==================================================================
        # Residential submodel load profiles
        # ==================================================================
        load_profiles = load_profile.LoadProfileStock("load_profiles")
        dummy_sector = None

        # --------Calculate HDD/CDD of used weather year
        self.rs_hdd_by, _ = hdd_cdd.calc_reg_hdd(
            temp_by, assumptions.t_bases.rs_t_heating, assumptions.model_yeardays, crit_temp_min_max)
        self.rs_hdd_cy, rs_fuel_shape_heating_yd = hdd_cdd.calc_reg_hdd(
            temp_cy, rs_t_base_heating_cy, assumptions.model_yeardays, crit_temp_min_max)

        # --------Calculate HDD/CDD of base year weather year
        #self.rs_cdd_by, _ = hdd_cdd.calc_reg_cdd(
        #    temp_by, assumptions.t_bases.rs_t_cooling_by, assumptions.model_yeardays)
        #self.rs_cdd_cy, rs_fuel_shape_cooling_yd = hdd_cdd.calc_reg_cdd(
        #    temp_cy, rs_t_base_cooling_cy, assumptions.model_yeardays)

        # -------Calculate climate change correction factors
        try:
            f_heat_rs_y = np.nan_to_num(
                1.0 / float(np.sum(self.rs_hdd_by))) * np.sum(self.rs_hdd_cy)
            #self.f_cooling_rs_y = np.nan_to_num(
            #    1.0 / float(np.sum(self.rs_cdd_by))) * np.sum(self.rs_cdd_cy)
            f_cooling_rs_y = 1
        except ZeroDivisionError:
            f_heat_rs_y = 1
            f_cooling_rs_y = 1

        # Calculate rs peak day
        rs_peak_day = enduse_func.get_peak_day(self.rs_hdd_cy)

        # ========
        # Enduse specific profiles
        # ========
        # -- Apply enduse sepcific shapes for enduses with not technologies with own defined shapes
        for enduse in enduses['residential']:

            # Enduses where technology specific load profiles are defined for yh
            if enduse in ['rs_space_heating']:
                pass
            else:
                # Get all technologies of enduse
                tech_list = helpers.get_nested_dict_key(
                    assumptions.fuel_tech_p_by[enduse][dummy_sector])

                # Remove heat pumps from rs_water_heating
                tech_list = basic_functions.remove_element_from_list(tech_list, 'heat_pumps_electricity')

                shape_y_dh = insert_peak_dh_shape(
                    peak_day=rs_peak_day,
                    shape_y_dh=tech_lp['rs_shapes_dh'][enduse]['shape_non_peak_y_dh'],
                    shape_peak_dh=tech_lp['rs_shapes_dh'][enduse]['shape_peak_dh'])

                load_profiles.add_lp(
                    unique_identifier=uuid.uuid4(),
                    technologies=tech_list,
                    enduses=[enduse],
                    shape_yd=tech_lp['rs_shapes_yd'][enduse]['shape_non_peak_yd'],
                    shape_y_dh=shape_y_dh,
                    sectors=[dummy_sector],
                    model_yeardays=assumptions.model_yeardays)

        # ==========
        # Technology specific profiles for residential heating
        # ===========
        # ------Heating boiler
        load_profiles.add_lp(
            unique_identifier=uuid.uuid4(),
            technologies=assumptions.tech_list['heating_const'],
            enduses=['rs_space_heating'],
            shape_yd=rs_fuel_shape_heating_yd,
            shape_y_dh=tech_lp['rs_profile_boilers_y_dh'],
            sectors=[dummy_sector],
            model_yeardays=assumptions.model_yeardays)

        # ------Heating CHP
        rs_profile_chp_y_dh = insert_peak_dh_shape(
            peak_day=rs_peak_day,
            shape_y_dh=tech_lp['rs_profile_chp_y_dh'],
            shape_peak_dh=tech_lp['rs_lp_heating_CHP_dh']['peakday'])

        load_profiles.add_lp(
            unique_identifier=uuid.uuid4(),
            technologies=assumptions.tech_list['tech_CHP'],
            enduses=['rs_space_heating'],
            shape_yd=rs_fuel_shape_heating_yd,
            shape_y_dh=rs_profile_chp_y_dh,
            sectors=[dummy_sector],
            model_yeardays=assumptions.model_yeardays)

        # ------Electric heating, storage heating (primary)
        load_profiles.add_lp(
            unique_identifier=uuid.uuid4(),
            technologies=assumptions.tech_list['storage_heating_electricity'],
            enduses=['rs_space_heating'],
            shape_yd=rs_fuel_shape_heating_yd,
            shape_y_dh=tech_lp['rs_profile_storage_heater_y_dh'],
            sectors=[dummy_sector],
            model_yeardays=assumptions.model_yeardays)

        # ------Electric heating secondary (direct elec heating)
        load_profiles.add_lp(
            unique_identifier=uuid.uuid4(),
            technologies=assumptions.tech_list['secondary_heating_electricity'],
            enduses=['rs_space_heating'],
            shape_yd=rs_fuel_shape_heating_yd,
            shape_y_dh=tech_lp['rs_profile_elec_heater_y_dh'],
            sectors=[dummy_sector],
            model_yeardays=assumptions.model_yeardays)

        # ------Heat pump heating
        rs_profile_hp_y_dh = insert_peak_dh_shape(
            peak_day=rs_peak_day,
            shape_y_dh=tech_lp['rs_profile_hp_y_dh'],
            shape_peak_dh=tech_lp['rs_lp_heating_hp_dh']['peakday'])

        rs_fuel_shape_hp_yh, rs_hp_shape_yd = get_fuel_shape_heating_hp_yh(
            tech_lp_y_dh=rs_profile_hp_y_dh,
            tech_stock=rs_tech_stock,
            rs_hdd_cy=self.rs_hdd_cy,
            model_yeardays=assumptions.model_yeardays)

        load_profiles.add_lp(
            unique_identifier=uuid.uuid4(),
            technologies=assumptions.tech_list['heating_non_const'],
            enduses=['rs_space_heating', 'rs_water_heating'],
            shape_y_dh=rs_profile_hp_y_dh,
            shape_yd=rs_hp_shape_yd,
            shape_yh=rs_fuel_shape_hp_yh,
            sectors=[dummy_sector],
            model_yeardays=assumptions.model_yeardays)

        # ------District_heating_electricity. Assumption made that same curve as CHP
        load_profiles.add_lp(
            unique_identifier=uuid.uuid4(),
            technologies=assumptions.tech_list['tech_district_heating'],
            enduses=['rs_space_heating'],
            shape_yd=rs_fuel_shape_heating_yd,
            shape_y_dh=rs_profile_hp_y_dh,
            sectors=[dummy_sector],
            model_yeardays=assumptions.model_yeardays)

        # ==================================================================
        # Service Submodel load profiles
        # ==================================================================
        # --------HDD/CDD
        # current weather_yr
        self.ss_hdd_by, _ = hdd_cdd.calc_reg_hdd(
            temp_by, assumptions.t_bases.ss_t_heating, assumptions.model_yeardays, crit_temp_min_max)

        ss_hdd_cy, ss_fuel_shape_heating_yd = hdd_cdd.calc_reg_hdd(
            temp_cy, ss_t_base_heating_cy, assumptions.model_yeardays, crit_temp_min_max)

        self.ss_cdd_by, _ = hdd_cdd.calc_reg_cdd(
            temp_by, assumptions.t_bases.ss_t_cooling, assumptions.model_yeardays, crit_temp_min_max)

        ss_cdd_cy, ss_lp_cooling_yd = hdd_cdd.calc_reg_cdd(
            temp_cy, ss_t_base_cooling_cy, assumptions.model_yeardays, crit_temp_min_max)

        try:
            f_heat_ss_y = np.nan_to_num(
                1.0 / float(np.sum(self.ss_hdd_by))) * np.sum(ss_hdd_cy)
            f_cooling_ss_y = np.nan_to_num(
                1.0 / float(np.sum(self.ss_cdd_by))) * np.sum(ss_cdd_cy)
        except ZeroDivisionError:
            f_heat_ss_y = 1
            f_cooling_ss_y = 1

        # ========
        # Enduse specific profiles
        # ========
        # - Assign to each enduse the carbon fuel trust dataset
        for enduse in enduses['service']:

            # Skip temperature dependent end uses (regional) because load profile in regional load profile stock
            if enduse in assumptions.enduse_space_heating or enduse in assumptions.ss_enduse_space_cooling:
                pass
            else:
                for sector in sectors['service']:

                    # Get technologies with assigned fuel shares
                    tech_list = helpers.get_nested_dict_key(
                        assumptions.fuel_tech_p_by[enduse][sector])

                    # Apply correction factor for weekend_effect
                    shape_non_peak_yd_weighted = load_profile.abs_to_rel(
                        tech_lp['ss_shapes_yd'][enduse][sector]['shape_non_peak_yd'] * assumptions.ss_weekend_f)

                    load_profiles.add_lp(
                        unique_identifier=uuid.uuid4(),
                        technologies=tech_list,
                        enduses=[enduse],
                        shape_yd=shape_non_peak_yd_weighted,
                        shape_y_dh=tech_lp['ss_shapes_dh'][enduse][sector]['shape_non_peak_y_dh'],
                        model_yeardays=assumptions.model_yeardays,
                        sectors=[sector])

        # Apply correction factor for weekend_effect for space heating
        ss_fuel_shape_heating_yd_weighted = load_profile.abs_to_rel(
            ss_fuel_shape_heating_yd * assumptions.ss_weekend_f)

        # ========
        # Technology specific profiles
        # ========

        # Flatten list of all potential technologies
        ss_space_heating_tech_lists = list(assumptions.tech_list.values())
        all_techs_ss_space_heating = [item for sublist in ss_space_heating_tech_lists for item in sublist]


        # -----Heat pump (RESIDENTIAL HEAT PUMP PROFILE FOR SERVICE SECTOR)
        all_techs_ss_space_heating = basic_functions.remove_element_from_list(
            all_techs_ss_space_heating, 'heat_pumps_electricity')

        ss_fuel_shape_hp_yh, ss_hp_shape_yd = get_fuel_shape_heating_hp_yh(
            tech_lp_y_dh=rs_profile_hp_y_dh,
            tech_stock=rs_tech_stock,
            rs_hdd_cy=ss_hdd_cy,
            model_yeardays=assumptions.model_yeardays)

        load_profiles.add_lp(
            unique_identifier=uuid.uuid4(),
            technologies=assumptions.tech_list['heating_non_const'],
            enduses=['ss_space_heating', 'ss_water_heating'],
            sectors=sectors['service'],
            shape_y_dh=rs_profile_hp_y_dh,
            shape_yd=ss_hp_shape_yd,
            shape_yh=ss_fuel_shape_hp_yh,
            model_yeardays=assumptions.model_yeardays)

        # ---secondary_heater_electricity Info: The residential direct heating load profile is used
        all_techs_ss_space_heating = basic_functions.remove_element_from_list(
            all_techs_ss_space_heating, 'secondary_heater_electricity')

        # Get aggregated electricity load profile
        #ALTERNATIVE :tech_lp['ss_all_tech_shapes_dh']['ss_other_electricity']['shape_non_peak_y_dh']
        load_profiles.add_lp(
            unique_identifier=uuid.uuid4(),
            technologies=assumptions.tech_list['secondary_heating_electricity'],
            enduses=['ss_space_heating'],
            sectors=sectors['service'],
            shape_yd=ss_fuel_shape_heating_yd_weighted,
            shape_y_dh=tech_lp['rs_profile_elec_heater_y_dh'],
            model_yeardays=assumptions.model_yeardays)
            # ELEC CURVE ss_fuel_shape_electricity # DIRECT HEATING ss_profile_elec_heater_yh

        # ---Heating technologies (all other)
        # (the heating shape follows the gas shape of aggregated sectors)
        # meaning that for all technologies, the load profile is the same
        load_profiles.add_lp(
            unique_identifier=uuid.uuid4(),
            technologies=all_techs_ss_space_heating,
            enduses=['ss_space_heating'],
            sectors=sectors['service'],
            shape_yd=ss_fuel_shape_heating_yd_weighted,
            shape_y_dh=tech_lp['ss_all_tech_shapes_dh']['ss_space_heating']['shape_non_peak_y_dh'],
            model_yeardays=assumptions.model_yeardays)

        # --Add cooling technologies for service sector
        coolings_techs = assumptions.tech_list['cooling_const']

        for cooling_enduse in assumptions.ss_enduse_space_cooling:
            for sector in sectors['service']:

                # Apply correction factor for weekend_effect 'cdd_weekend_cfactors'
                ss_lp_cooling_yd_weighted = load_profile.abs_to_rel(
                    ss_lp_cooling_yd * assumptions.cdd_weekend_cfactors)

                load_profiles.add_lp(
                    unique_identifier=uuid.uuid4(),
                    technologies=coolings_techs,
                    enduses=[cooling_enduse],
                    sectors=[sector],
                    shape_yd=ss_lp_cooling_yd_weighted,
                    shape_y_dh=tech_lp['ss_profile_cooling_y_dh'],
                    model_yeardays=assumptions.model_yeardays)

        # ==================================================================
        # Industry submodel load profiles
        # ==================================================================

        # --------HDD/CDD
        # Current weather_yr
        self.is_hdd_by, _ = hdd_cdd.calc_reg_hdd(
            temp_by, assumptions.t_bases.is_t_heating, assumptions.model_yeardays, crit_temp_min_max)

        #is_cdd_by, _ = hdd_cdd.calc_reg_cdd(
        #    temp_by, assumptions.t_bases.is_t_cooling, assumptions.model_yeardays)

        # Take same base temperature as for service sector
        is_hdd_cy, is_fuel_shape_heating_yd = hdd_cdd.calc_reg_hdd(
            temp_cy, is_t_base_heating_cy, assumptions.model_yeardays, crit_temp_min_max)
        #is_cdd_cy, _ = hdd_cdd.calc_reg_cdd(
        #    temp_cy, ss_t_base_cooling_cy, assumptions.model_yeardays)

        try:
            f_heat_is_y = np.nan_to_num(1.0 / float(np.sum(self.is_hdd_by))) * np.sum(is_hdd_cy)
            #self.f_cooling_is_y = np.nan_to_num(1.0 / float(np.sum(is_cdd_by))) * np.sum(is_cdd_cy)
            f_cooling_is_y = 1
        except ZeroDivisionError:
            f_heat_is_y = 1
            f_cooling_is_y = 1

        # ========
        # Technology specific profiles
        # ========

        # --Heating technologies

        # Flatten list of all potential heating technologies
        is_space_heating_tech_lists = list(assumptions.tech_list.values())
        all_techs_is_space_heating = [item for sublist in is_space_heating_tech_lists for item in sublist]

        # Apply correction factor for weekend_effect for space heating load profile
        is_fuel_shape_heating_yd_weighted = load_profile.abs_to_rel(
            is_fuel_shape_heating_yd * assumptions.is_weekend_f)

        # - Direct electric heating
        # Remove tech from all space heating techs
        all_techs_is_space_heating = basic_functions.remove_element_from_list(
            all_techs_is_space_heating, 'secondary_heater_electricity')

        load_profiles.add_lp(
            unique_identifier=uuid.uuid4(),
            technologies=assumptions.tech_list['secondary_heating_electricity'],
            enduses=['is_space_heating'],
            sectors=sectors['industry'],
            shape_yd=is_fuel_shape_heating_yd_weighted,
            shape_y_dh=tech_lp['rs_profile_elec_heater_y_dh'],
            model_yeardays=assumptions.model_yeardays)

        # Add flat load profiles for non-electric heating technologies
        load_profiles.add_lp(
            unique_identifier=uuid.uuid4(),
            technologies=all_techs_is_space_heating,
            enduses=['is_space_heating'],
            sectors=sectors['industry'],
            shape_yd=is_fuel_shape_heating_yd_weighted,
            shape_y_dh=flat_shape_y_dh,
            model_yeardays=assumptions.model_yeardays)

        # Apply correction factor for weekend_effect to flat load profile for industry
        flat_shape_yd = flat_shape_yd * assumptions.is_weekend_f
        flat_shape_yd_weighted = load_profile.abs_to_rel(flat_shape_yd)

        # ========
        # Enduse specific profiles
        # ========
        for enduse in enduses['industry']:
            if enduse == "is_space_heating":
                pass # Do not create non regional stock because temp dependent
            else:
                for sector in sectors['industry']:

                    tech_list = helpers.get_nested_dict_key(
                        assumptions.fuel_tech_p_by[enduse][sector])

                    load_profiles.add_lp(
                        unique_identifier=uuid.uuid4(),
                        technologies=tech_list,
                        enduses=[enduse],
                        shape_yd=flat_shape_yd_weighted,
                        shape_y_dh=flat_shape_y_dh,
                        model_yeardays=assumptions.model_yeardays,
                        sectors=[sector])

        # ---------------
        # Load profiles
        # ---------------
        self.load_profiles = load_profiles

        self.f_heat = {
            'residential': f_heat_rs_y,
            'service': f_heat_ss_y,
            'industry': f_heat_is_y}

        self.f_colling = {
            'residential': f_cooling_rs_y,
            'service': f_cooling_ss_y,
            'industry': f_cooling_is_y}

def get_fuel_shape_heating_hp_yh(tech_lp_y_dh, tech_stock, rs_hdd_cy, model_yeardays):
    """Convert daily shapes to houly based on load for heatpump

    This is for non-peak.

    Arguments
    ---------
    tech_lp_y_dh : dict
        Technology load profiles
    tech_stock : object
        Technology stock
    rs_hdd_cy : array
        Heating Degree Days
    model_yeardays : array
        Modelled year days

    Returns
    -------
    shape_yh : array
        Yearly shape to calculate hourly load (total sum == 1)
    hp_yd : array
        Yd shape

    Note
    ----
    -  An average heat pump efficiency is calculated for the whole day
       depending on hourly temperatures.

    -  See XY in documentation for source of heat pumps
    """
    shape_yh_hp = np.zeros((365, 24), dtype="float")
    shape_y_dh = np.zeros((365, 24), dtype="float")

    tech_eff = tech_stock.get_tech_attr(
        'rs_space_heating',
        None,
        'heat_pumps_electricity',
        'eff_cy')

    # Convert daily service demand to fuel (fuel = Heat demand / efficiency)
    # As only the shape is of interest, the HDD
    # can be used as an indicator for fuel use (which correlates) directly
    hp_yd = rs_hdd_cy[:, np.newaxis] / tech_eff

    # Distribute daily according to fuel load curves of heat pumps
    shape_yh_hp = hp_yd * tech_lp_y_dh

    # Convert absolute hourly fuel demand to relative fuel demand within a year
    shape_yh = load_profile.abs_to_rel(shape_yh_hp)

    # Convert for every day the shape to absolute shape (tot sum for a full year == 365)
    _shape_y_dh_sum_rows = np.sum(shape_yh_hp, axis=1)

    with np.errstate(divide='ignore', invalid='ignore'):
        shape_y_dh = shape_yh_hp / _shape_y_dh_sum_rows[:, np.newaxis]
    shape_y_dh[np.isnan(shape_y_dh)] = 0

    # Select only modelled days
    return shape_yh[model_yeardays], hp_yd

def get_shape_cooling_yh(tech_shape, cooling_shape):
    """Convert daily shape to hourly

    Arguments
    ---------
    tech_shape : dict
        Technology shape
    cooling_shape : array
        Cooling profile

    Returns
    -------
    shape_yd_cooling_tech : array
        Shape of cooling devices

    Note
    ----
    The daily cooling demand (calculated with cdd) is distributed within the day
    fuel demand curve from:

    - **Residential**: Taken from *Denholm, P., Ong, S., & Booten, C. (2012).
        Using Utility Load Data to Estimate Demand for Space Cooling and
        Potential for Shiftable Loads, (May), 23.
        Retrieved from http://www.nrel.gov/docs/fy12osti/54509.pdf*

    - **Service**: *Knight, Dunn, Environments Carbon and Cooling in
        Uk Office Environments*
    """
    shape_yd_cooling_tech = tech_shape * cooling_shape[:, np.newaxis]

    return shape_yd_cooling_tech

def ss_get_sector_enduse_shape(tech_lps, heating_lp_yd, enduse):
    """Read generic shape for all technologies in a service sector enduse

    Arguments
    ---------
    tech_lps : array
        Technology load profiles
    heating_lp_yd : array
        Daily (yd) service demand shape for heat (percentage of yearly
        heat demand for every day)
    enduse : str
        Enduse where technology is used

    Returns
    -------
    shape_boilers_yh : array
        Shape how yearly fuel can be distributed to hourly (yh) (total sum == 1)
    shape_boilers_y_dh : array
        Shape of distribution of fuel within every day of a year (total sum == nr_of_days)
    """
    shape_yh_generic_tech = np.zeros((365, 24), dtype="float")

    if enduse not in tech_lps:
        pass
    else:
        shape_y_dh_generic_tech = tech_lps[enduse]['shape_non_peak_y_dh']
        shape_yh_generic_tech = heating_lp_yd[:, np.newaxis] * shape_y_dh_generic_tech

    return shape_yh_generic_tech, shape_y_dh_generic_tech

def insert_peak_dh_shape(
        peak_day,
        shape_y_dh,
        shape_peak_dh
    ):
    """Insert peak specific load profile of a technology
    Arguments
    ---------
    peak_day : int
        Peak day nr
    shape_y_dh : array
        Shape of technology for every day (total sum = 365)
    tech_lp_tech : dict
        Technolgy specific load profiles for different day types
    Returns
    -------
    shape_y_dh_inserted : array
        Array where on peak day the peak shape is inserted
    """
    shape_y_dh_inserted = np.copy(shape_y_dh)

    shape_y_dh_inserted[peak_day] = shape_peak_dh

    return shape_y_dh_inserted

def get_weather_station_selection(
        all_weather_stations,
        counter,
        weather_yr
    ):
    """Select weather stations based on
    position in list

    Arguments
    --------
    counter : int
        Count to get weather station nr
    """
    all_stations_of_weather_yr = list(all_weather_stations[weather_yr].keys())

    # Sort
    all_stations_of_weather_yr.sort()

    try:
        # Select station according ot position in list
        station_id = all_stations_of_weather_yr[counter]

        all_weather_stations_out = {station_id: all_weather_stations[weather_yr][station_id]}
    except KeyError:
        # Not enough stations to select position in list
        station_id = False
        all_weather_stations_out = []
        logging.debug("... no weather station found")

    return all_weather_stations_out, station_id



"""Functions handling the narratives
"""
import logging
from collections import defaultdict

from energy_demand.basic import lookup_tables


def get_all_sectors_of_narratives(narratives):
    """Get all defined sectors of all narratives

    ARguments
    --------
    narratives : list
        All defined narratives

    Returns
    -------
    all_sectors : list
        All sectors
    """
    all_sectors = set()
    for narrative in narratives:
        all_sectors.add(narrative['sector'])
    all_sectors = list(all_sectors)

    return all_sectors


def get_sector_narrative_and_single_from_multi(sector_to_match, switches):
    """Get all switches of a sector if the switches are
    defined specifically for a sector. If the switches are
    not specifically for a sector, return all switches

    Arguments
    ----------
    sector_to_match : int
        Sector to find switches
    switches : list
        Switches

    Returns
    -------
    switches : list
        Switches of sector
    """
    if sector_to_match is True:
        return switches
    else:
        switches_out = []

        # Test if multiple switches e.g. per fueltype
        try:
            fueltypes_switched = set([])
            for switch in switches:
                fueltypes_switched.add(switch['fueltype_new'])
        except:
            fueltypes_switched = []

        if len(fueltypes_switched) > 1:

            for fueltype in fueltypes_switched:
                switches_out_single = []
                for switch in switches:
                    if switch['sector'] == sector_to_match and switch['fueltype_new'] == fueltype:
                        switches_out_single.append(switch)
                    elif not switch['sector'] and switch['fueltype_new'] == fueltype: # Not defined specifically for sectors and append all
                        switches_out_single.append(switch)
                    else:
                        pass
                switches_out.append(switches_out_single)
        else:
            switches_out_single = []
            for switch in switches:
                if switch['sector'] == sector_to_match:
                    switches_out_single.append(switch)
                elif not switch['sector']: # Not defined specifically for sectors and append all
                    switches_out_single.append(switch)
                else:
                    pass

            switches_out = [switches_out_single]

        return switches_out

def crit_dim_var(var):
    """Check if nested dict or not

    Arguments
    ---------
    var : dict
        Dictionary to test wheter single or multidimensional parameter

    Returns
    -------
    single_dimension : bool
        True: Single dimension, False: Multidimensional parameter
    """
    single_dimension = True

    exclude = ['regional_vals_by', 'regional_vals_ey']

    # Test if list nor not
    if type(var) is list:
        for list_entry in var:
            for key, value in list_entry.items():

                if type(value) is not list:
                    if hasattr(value, 'keys') and key not in exclude:
                        if len(value.keys()) != 0:
                            single_dimension = False
                        else:
                            pass
    else:
        for key, value in var.items():
            if type(value) is not list:
                if hasattr(value, 'keys') and key not in exclude:
                    if len(value.keys()) != 0:
                        single_dimension = False
            else:
                if value == []:
                    pass
                else:
                    single_dimension = False

    return single_dimension

def read_from_narrative(narratives):
    """Read from narratives the defined
    value for the last defined timestep

    Arguments
    ---------
    narratives : lives
        Narratives

    Returns
    -------
    last_value : float
        Value of last defined timestep narrative
    """
    last_year = 0
    for narrative in narratives:
        if narrative['end_yr'] > last_year:
            last_value = narrative['value_ey']
            last_year = narrative['end_yr']

    return last_value

def default_narrative(
        end_yr,
        value_by,
        value_ey,
        diffusion_choice='linear',
        sig_midpoint=0,
        sig_steepness=1,
        base_yr=2015,
        regional_specific=True,
        fueltype_replace=0,
        fueltype_new=0,
    ):
    """Create a default single narrative with a single timestep

    E.g. from value 0.2 in 2015 to value 0.5 in 2050

    Arguments
    ----------
    end_yr : int
        End year of narrative
    value_by : float
        Value of start year of narrative
    value_ey : float
        Value at end year of narrative
    diffusion_choice : str, default='linear'
        Wheter linear or sigmoid
    sig_midpoint : float, default=0
        Sigmoid midpoint
    sig_steepness : float, default=1
        Sigmoid steepness
    base_yr : int
        Base year
    regional_specific : bool
        If regional specific or not

    Returns
    -------
    container : list
        List with narrative
    """
    return [{
        'base_yr': base_yr,
        'end_yr': end_yr,
        'value_by': value_by,
        'value_ey': value_ey,
        'diffusion_choice': diffusion_choice,
        'sig_midpoint': sig_midpoint,
        'sig_steepness': sig_steepness,
        'regional_specific': regional_specific,
        'fueltype_replace': fueltype_replace,
        'fueltype_new': fueltype_new
        }]

def autocomplete(parameter_narratives, simulation_base_yr, sub_param_crit):
    """
    """
    autocomplet_param_narr = defaultdict(dict)

    for sub_param_name, narratives_sector in parameter_narratives.items():
        logging.debug("   ... " + str(sub_param_name))
        for sector, narratives in narratives_sector.items():
            autocomplet_param_narr[sub_param_name][sector] = []

            switches_to_create_narrative = get_sector_narrative_and_single_from_multi(
                sector, narratives)

            for switch_to_create_narrative in switches_to_create_narrative:
                # Get all years of switches_to_create_narrative
                all_yrs = []
                for narrative in switch_to_create_narrative:
                    all_yrs.append(narrative['end_yr'])

                all_yrs.sort()

                for year_cnt, year in enumerate(all_yrs):
                    for narrative in switch_to_create_narrative:
                        if narrative['end_yr'] == year:
                            yr_narrative = narrative
                            break

                    # Add missing entries to narrative
                    if year_cnt == 0:
                        # Update
                        yr_narrative['base_yr'] = simulation_base_yr
                        yr_narrative['value_by'] = narrative['default_by']

                        # previous value
                        previous_yr = narrative['end_yr']
                        previous_value = narrative['value_ey']
                    else:
                        # Update
                        yr_narrative['base_yr'] = previous_yr
                        yr_narrative['value_by'] = previous_value

                        # previous value
                        previous_yr = narrative['end_yr']
                        previous_value = narrative['value_ey']

                    autocomplet_param_narr[sub_param_name][sector].append(yr_narrative)

    # Remove all dummy sector
    autocomplet_param_narr_new = defaultdict(dict)

    for param_name, sector_data in autocomplet_param_narr.items():
        for sector, data in sector_data.items():
            if sector == 'dummy_sector':
                autocomplet_param_narr_new[param_name] = data
            else:
                autocomplet_param_narr_new[param_name][sector] = data

    autocomplet_param_narr = dict(autocomplet_param_narr_new)

    # If only single dimension parameter, remove dummy mutliparameter name
    if not sub_param_crit:
        try:
            autocomplet_param_narr = autocomplet_param_narr['dummy_single_param']
        except:
            pass

    return autocomplet_param_narr

def read_user_defined_param(
        df,
        simulation_base_yr,
        simulation_end_yr,
        default_streategy_var,
        var_name
    ):
    """Read in user defined narrative parameters
    """
    parameter_narratives = {}
    single_param_narratives = {}

    lookups = lookup_tables.basic_lookups()
    columns = list(df.columns)

    if 'enduses' in columns:
        sub_param_crit = True
    else:
        sub_param_crit = False

    if len(list(df.columns)) == 1:
        single_dim_param = True
    else:
        single_dim_param = False

    if single_dim_param:

        # Read single dimension param and create single step narrative
        single_step_narrative = {}
        single_step_narrative['sector'] = 'dummy_sector'
        single_step_narrative['default_value'] = default_streategy_var['default_value']
        single_step_narrative['value_ey'] = float(df[var_name][0])
        single_step_narrative['end_yr'] = simulation_end_yr
        single_step_narrative['base_yr'] = simulation_base_yr
        single_step_narrative['value_by'] = default_streategy_var['default_value']
        single_step_narrative['regional_specific'] = default_streategy_var['regional_specific']
        single_step_narrative['diffusion_choice'] = 'linear'
        single_param_narratives = [single_step_narrative]
        return single_param_narratives
    else:
        # Read multidmensional param
        if sub_param_crit:
            enduses = set(df['enduses'].values)
            for enduse in enduses:
                parameter_narratives[enduse] = {}

                # All rows of enduse
                df_enduse = df.loc[df['enduses'] == enduse]

                # Get all sectors and years
                sectors = set()
                end_yrs = set()

                for i in df_enduse.index:
                    try:
                        sector = df_enduse.at[i, 'sector']
                        sectors.add(sector)
                    except:
                        pass

                    try:
                        end_yr = int(df_enduse.at[i, 'end_yr'])
                        end_yrs.add(end_yr)
                    except:
                        pass
                if list(sectors) == []:

                    for end_yr in end_yrs:
                        try:
                            _ = default_streategy_var[enduse]
                            defined_in_model = True
                        except KeyError:
                            defined_in_model = False

                        # All entries of this year df_enduse and this fueltype
                        df_enduse_sim_yr = df_enduse.loc[df_enduse['end_yr'] == end_yr]

                        if defined_in_model:
                            narrative = {}
                            narrative['sector'] = 'dummy_sector'
                            narrative['end_yr'] = end_yr
                            narrative['sig_midpoint'] = 0
                            narrative['sig_steepness'] = 1
                            narrative['regional_specific'] = default_streategy_var[enduse]['regional_specific']
                            narrative['default_by'] = default_streategy_var[enduse]['default_value']

                            # Check if more than one entry 
                            for _index, row in df_enduse_sim_yr.iterrows():

                                try:
                                    interpolation_params = row['interpolation_params']
                                except KeyError:
                                    # Generic fuel switch
                                    interpolation_params = row['param_generic_fuel_switch']

                                # If more than one switch per enduse
                                if interpolation_params in narrative:

                                    # Add narrative and start new one
                                    try:
                                        parameter_narratives[enduse][narrative['sector']].append(narrative)
                                    except KeyError:
                                        parameter_narratives[enduse][narrative['sector']] = [narrative]

                                    narrative = {}
                                    narrative['sector'] = 'dummy_sector'
                                    narrative['end_yr'] = end_yr
                                    narrative['sig_midpoint'] = 0
                                    narrative['sig_steepness'] = 1
                                    narrative['regional_specific'] = default_streategy_var[enduse]['regional_specific']
                                    narrative['default_by'] = default_streategy_var[enduse]['default_value']

                                    if interpolation_params == 'diffusion_choice':
                                        int_diffusion_choice = float(row[var_name])
                                        narrative['diffusion_choice'] = lookups['diffusion_type'][int_diffusion_choice]
                                    else:
                                        narrative[interpolation_params] = float(row[var_name])
                                else:
                                    if interpolation_params == 'diffusion_choice':
                                        int_diffusion_choice = float(row[var_name])
                                        narrative['diffusion_choice'] = lookups['diffusion_type'][int_diffusion_choice]
                                    else:
                                        narrative[interpolation_params] = float(row[var_name])

                            # Add narrative
                            try:
                                parameter_narratives[enduse][narrative['sector']].append(narrative)
                            except KeyError:
                                parameter_narratives[enduse][narrative['sector']] = [narrative]
                else:
                    # If setctor specific, this needs to be implemented
                    pass
        else:
            sectors = set()
            end_yrs = set()

            for i in df.index:
                try:
                    sector = df.at[i, 'sector']
                    sectors.add(sector)
                except:
                    pass
                try:
                    end_yr = int(df.at[i, 'end_yr'])
                    end_yrs.add(end_yr)
                except:
                    pass

            if list(sectors) == []:
                parameter_narratives['dummy_single_param'] = {}

                for end_yr in end_yrs:
                    narrative = {}
                    narrative['sector'] = 'dummy_sector'
                    narrative['end_yr'] = end_yr
                    narrative['sig_midpoint'] = 0
                    narrative['sig_steepness'] = 1
                    narrative['regional_specific'] = default_streategy_var['regional_specific']
                    narrative['default_by'] = default_streategy_var['default_value']

                    for _index, row in df.iterrows():

                        interpolation_params = row['interpolation_params']

                        if interpolation_params == 'diffusion_choice':
                            lookups = lookup_tables.basic_lookups()
                            int_diffusion_choice = int(row[var_name])
                            narrative['diffusion_choice'] = lookups['diffusion_type'][int_diffusion_choice]
                        else:
                            narrative[interpolation_params] = float(row[var_name])

                    # Add narrative
                    try:
                        parameter_narratives['dummy_single_param'][narrative['sector']].append(narrative)
                    except (KeyError, AttributeError):
                        parameter_narratives['dummy_single_param'][narrative['sector']] = [narrative]
            else:
                # Needs to be implemented in case sector specific
                pass

        # ------------
        # Autocomplete
        # ------------
        logging.debug("... autocomplete")
        parameter_narratives = autocomplete(
            parameter_narratives,
            simulation_base_yr,
            sub_param_crit)

        return parameter_narratives



"""Provides an entry point from the command line to the energy demand model
"""
import os
import sys
from argparse import ArgumentParser
from pkg_resources import Requirement
from pkg_resources import resource_filename
import energy_demand
from energy_demand.read_write import data_loader
from energy_demand.assumptions import general_assumptions
from energy_demand.scripts.s_post_installation import post_install_setup
from energy_demand.scripts.s_write_dummy_data import post_install_setup_minimum
from energy_demand.scripts.init_scripts import switch_calculations
from energy_demand.read_write import read_data
from energy_demand.dwelling_stock import dw_stock
from energy_demand.plotting import plotting_results
from energy_demand.basic import basic_functions
from energy_demand.basic import date_prop

def parse_arguments():
    """Parse command line arguments

    Returns
    =======
    :class:`argparse.ArgumentParser`

    """
    parser = ArgumentParser(description='Command line tools for energy_demand')
    parser.add_argument('-V', '--version',
                        action='version',
                        version="energy_demand" + energy_demand.__version__,
                        help='show the current version of energy_demand')

    subparsers = parser.add_subparsers()

    # Initialisation of energy demand model (processing raw files)
    parser_init = subparsers.add_parser(
        'setup',
        help='Executes the raw reading functions')

    parser_init.add_argument(
        '-f',
        '--config_file',
        default='./data',
        help='Path to the local config file')

    parser_init.set_defaults(func=post_install_setup)

    # Initialisation of energy demand model (processing raw files)
    parser_init2 = subparsers.add_parser(
        'minimal_setup',
        help='Executes the minimum dummy raw reading functions')

    parser_init2.add_argument(
        '-f',
        '--config_file',
        default='./data',
        help='Path to the local config file')

    parser_init2.set_defaults(func=post_install_setup_minimum)

    return parser

def main(arguments=None):
    """Parse args and run
    """
    parser = parse_arguments()
    args = parser.parse_args(arguments)

    if 'func' in args:
        args.func(args)
    else:
        parser.print_help()

if __name__ == '__main__':
    main(sys.argv[1:])



"""Generate results for paper 3


Collect model run and create folder with follwing structure:

scenario / fueltype /       type           /   year.csv
                        average,p_05,           contains 8760 h
                
                


data.csv

                region_a    region_b    ...

    hour0
    hour1    
    ...
"""
import os
import numpy as np
import pandas as pd
import copy

from energy_demand.read_write import data_loader
from energy_demand.technologies import tech_related
from energy_demand.basic import date_prop, basic_functions
from energy_demand import enduse_func

# Folder paths
path_out = "C:/__DATA_RESULTS_FINAL"                          # Folder to store results
path_results = "//linux-filestore.ouce.ox.ac.uk/mistral/nismod/data/energy_demand/_p3_weather_final"

# Scenario definitions
all_scenarios = ['h_max']#, 'h_min', 'l_max', 'l_min']
fueltypes = ['electricity', 'gas', 'hydrogen']
folder_types = ['mean', 'pos_two_sigma', 'neg_two_sigma']
simulation_yrs = range(2015, 2051, 5)

# -----------------------
# Create folder structure
# -----------------------
basic_functions.create_folder(path_out)
for scenario in all_scenarios:
    basic_functions.create_folder(os.path.join(path_out, scenario))
    for fueltype in fueltypes:
        basic_functions.create_folder(os.path.join(path_out, scenario, fueltype))
        for folder_type in folder_types:
            basic_functions.create_folder(os.path.join(path_out, scenario, fueltype, folder_type))
print("Created folder structure")

# ----------------------
# Write to file
# ---------------------
for scenario in all_scenarios:
    all_realizations = os.listdir(os.path.join(path_results, scenario))
    print("...scenario {}".format(scenario), flush=True)

    for simulation_yr in simulation_yrs:

        print("     ...simulation_yr {}".format(simulation_yr), flush=True)
        # ----------------------------------
        # Container to load all realizations initially for speed up
        # ----------------------------------
        container_all_initialisations = []
        for initialization in all_realizations:

            path_sim_yr = os.path.join(
                path_results,
                scenario,
                initialization,
                "simulation_results",
                "model_run_results_txt",
                "only_fueltype_reg_8760",
                "fueltype_reg_8760__{}.npy".format(simulation_yr))
            print("       ... loading scenario: {}".format(initialization), flush=True)

            full_result = np.load(path_sim_yr)
            container_all_initialisations.append(full_result)

            # Check peak
            fueltype_int = tech_related.get_fueltype_int('electricity')
            national_hourly_demand = np.sum(full_result[fueltype_int], axis=0)
            peak_day_electricity, _ = enduse_func.get_peak_day_single_fueltype(national_hourly_demand)
            selected_hours = date_prop.convert_yearday_to_8760h_selection(peak_day_electricity)
            print("PEAK electricity: " + str(np.max(national_hourly_demand[selected_hours])))

        for fueltype_str in fueltypes:
            print("         ...fueltype {}".format(fueltype_str), flush=True)
            fueltype_int = tech_related.get_fueltype_int(fueltype_str)

            # --------
            # Calculate 
            # --------
            path_ini_file = os.path.join(path_results, scenario, all_realizations[0])
            _, _, regions = data_loader.load_ini_param(path_ini_file)
            hours = range(8760)

            df_result_final_mean = pd.DataFrame(index=hours, columns=regions)
            df_result_pos_two_sigma = pd.DataFrame(index=hours, columns=regions)
            df_result_neg_two_sigma = pd.DataFrame(index=hours, columns=regions)

            df_result_region_empty = pd.DataFrame(columns=hours)

            for region_nr, reg_name in enumerate(regions):
                #print("         ...calculating region {}".format(reg_name), flush=True)

                #df_result_region = pd.DataFrame(columns=hours)
                df_result_region = copy.copy(df_result_region_empty)

                '''for cnt, initialization in enumerate(all_realizations):

                    path_sim_yr = os.path.join(
                        path_results,
                        scenario,
                        initialization,
                        "simulation_results",
                        "model_run_results_txt",
                        "only_fueltype_reg_8760",
                        "fueltype_reg_8760__{}.npy".format(simulation_yr))

                    full_result = np.load(path_sim_yr)'''

                for cnt, full_result in enumerate(container_all_initialisations):
                    result_reg_8760 = list(full_result[fueltype_int][region_nr])

                    #Slow
                    #realisation_data = pd.DataFrame([result_reg_8760], columns=hours)
                    #df_result_region = df_result_region.append(realisation_data)

                    #Faster
                    df_result_region.loc[cnt] = result_reg_8760

                # Calculate statistics
                reg_mean = df_result_region.mean() # Calculate mean of region
                std_dev = df_result_region.mean() # Calculate mean of region
                #reg_05 = df_result_region.quantile(q=0.05) # Calculate 0.05 quantiles of region
                #reg_95 = df_result_region.quantile(q=0.95) # Calculate 0.95 quantiles of region

                # Number of sigma
                nr_of_sigma = 2
                pos_two_sigma = reg_mean + (nr_of_sigma * std_dev)
                neg_two_sigma = reg_mean - (nr_of_sigma * std_dev)

                # Store in final dataframe
                df_result_final_mean[reg_name] = reg_mean
                df_result_pos_two_sigma[reg_name] = pos_two_sigma
                df_result_neg_two_sigma[reg_name] = neg_two_sigma

                #df_result_final_05[reg_name] = reg_05
                #df_result_final_95[reg_name] = reg_95
        
            # -------------
            # Write results
            # -------------
            out_path_file_mean = os.path.join(path_out, scenario, fueltype_str, 'mean', "{}.csv".format(simulation_yr))
            out_path_file_05 = os.path.join(path_out, scenario, fueltype_str, 'pos_two_sigma', "{}.csv".format(simulation_yr))
            out_path_file_95 = os.path.join(path_out, scenario, fueltype_str, 'neg_two_sigma', "{}.csv".format(simulation_yr))

            df_result_final_mean.to_csv(out_path_file_mean)
            df_result_pos_two_sigma.to_csv(out_path_file_05)
            df_result_neg_two_sigma.to_csv(out_path_file_95)
            #df_result_final_05.to_csv(out_path_file_05)
            #df_result_final_95.to_csv(out_path_file_95)

print("--------")
print("Finished writing out simulation results")
print("--------")


import os
import subprocess
from multiprocessing import Pool, cpu_count
import numpy as np
from energy_demand.read_write import read_weather_data

def my_function(simulation_number):  
    print('simulation_number ' + str(simulation_number))

    # Run smif
    run_commands = [
        "smif run energy_demand_constrained_A",
        "smif run energy_demand_constrained_A_dm_water",
        "smif run energy_demand_constrained_A_dm_water_space",

        "smif run energy_demand_constrained_B",
        "smif run energy_demand_constrained_B_dm_water",
        "smif run energy_demand_constrained_B_dm_water_space",

        "smif run energy_demand_constrained_C",
        "smif run energy_demand_constrained_C_dm_water",
        "smif run energy_demand_constrained_C_dm_water_space",

        "smif run energy_demand_constrained_D",
        "smif run energy_demand_constrained_D_dm_water",
        "smif run energy_demand_constrained_D_dm_water_space"]

    os.system(run_commands[simulation_number])

    return

simulation_number = range(1) #all scenarios

if __name__ == "__main__":
    with Pool(int(cpu_count()/2)) as pool:
        pool.map(
            my_function,
            simulation_number,
            chunksize=1)

'''
for i in range(2):


    # Activate virtual environement
    bashCommand = "activate ed"
    os.system(bashCommand)

    # Run smif
    bashCommand = "smif -v run ed_constrained_pop-baseline16_econ-c16_fuel-c16"
    os.system(bashCommand)

    #process = subprocess.Popen(bashCommand.split(), stdout=subprocess.PIPE)
    #output, error = process.communicate()
'''
'''

#import this
from multiprocessing import Pool, cpu_count

#import any other packages 
import numpy as np


def         my_function(simulation_number):  
print('simulation_number')
                return 


simulation_list = [1,2,3,4,5,6,7,8,9,10]

if __name__ == "__main__":
    with Pool(int(cpu_count()/2)) as pool:
         pool.map(my_function,simulation_list,chunksize=1)
'''

'''
#import sh
#sh.cd('C:/Users/cenv0553/ed')

#print(sh.pwd())

#stream = os.popen("cd C:/Users/cenv0553/ed")

'''


"""Generate charts from multiple scenarios
"""
import os
from energy_demand.read_write import read_data
from energy_demand.basic import date_prop
from energy_demand.plotting import plotting_multiple_scenarios
from energy_demand.basic import basic_functions
from energy_demand.basic import lookup_tables
from energy_demand.read_write import data_loader

def process_scenarios(path_to_scenarios, year_to_model=2015):
    """Iterate folder with scenario results and plot charts

    Arguments
    ----------
    path_to_scenarios : str
        Path to folders with stored results
    year_to_model : int, default=2015
        Year of base year
    """
    # -----------
    # Charts to plot
    # -----------
    heat_pump_range_plot = True        # Plot of changing scenario values stored in scenario name

    # Delete folder results if existing
    path_result_folder = os.path.join(
        path_to_scenarios, "__results_hp_chart")

    basic_functions.delete_folder(path_result_folder)

    seasons = date_prop.get_season(
        year_to_model=year_to_model)

    model_yeardays_daytype, _, _ = date_prop.get_yeardays_daytype(
        year_to_model=year_to_model)

    lookups = lookup_tables.basic_lookups()

    # Get all folders with scenario run results (name of folder is scenario)
    scenarios_hp = os.listdir(path_to_scenarios)

    scenario_data = {}

    for scenario_hp in scenarios_hp:
        print("HP SCENARIO " + str(scenario_hp))
        print(path_to_scenarios)
        scenario_data[scenario_hp] = {}

        # Simulation information is read in from .ini file for results
        path_fist_scenario = os.path.join(path_to_scenarios, scenario_hp)

        # -------------------------------
        # Iterate folders and get results
        # -------------------------------
        scenarios = os.listdir(path_fist_scenario)

        for scenario in scenarios:

            enduses, assumptions, reg_nrs, regions = data_loader.load_ini_param(
                os.path.join(path_fist_scenario, scenario))

            # Add scenario name to folder
            scenario_data[scenario_hp][scenario] = {}

            path_to_result_files = os.path.join(
                path_fist_scenario,
                scenario,
                'model_run_results_txt')

            scenario_data[scenario_hp][scenario] = read_data.read_in_results(
                path_result=path_to_result_files,
                seasons=seasons,
                model_yeardays_daytype=model_yeardays_daytype)

    # -----------------------
    # Generate result folder
    # -----------------------
    basic_functions.create_folder(path_result_folder)

    # -------------------------------
    # Generate plot with heat pump ranges
    # -------------------------------
    if heat_pump_range_plot:

        plotting_multiple_scenarios.plot_heat_pump_chart_multiple(
            lookups,
            regions,
            hp_scenario_data=scenario_data,
            fig_name=os.path.join(path_result_folder, "comparison_hp_share_peak_h.pdf"),
            txt_name=os.path.join(path_result_folder, "comparison_hp_share_peak_h.txt"),
            fueltype_str_input='electricity',
            plotshow=True)

    return

# Generate plots across all scenarios
process_scenarios(os.path.abspath("C:/Users/cenv0553/ed/results/Fig_12_multi_hp"))


"""Functions related to heating or cooling degree days
"""
import numpy as np

from energy_demand.basic import testing_functions
from energy_demand.geography import weather_station_location as weather_station
from energy_demand.profiles import load_profile

def calc_cdd_min_max(t_min, t_max, t_base):
    """Calculate daily cdd with minimum and maximum
    daily temperatures

    Arguments
    ---------
    t_min : array, shape(365)
        Daily minimum temperatures
    t_max : array, shape(365)
        Daily maximum temperatures
    t_base : float
        Base temperature

    Returns
    -------
    cdd_365 : array
        Daily heating degree days
    """
    cdd_365 = np.zeros((365))
    for day in range(365):

        t_min_day = t_min[day]
        t_max_day = t_max[day]

        case_nr = get_meterological_equation_case_cdd(
            t_min_day,
            t_max_day,
            t_base)

        cdd = calc_met_equation_cdd(
            case_nr,
            t_min_day,
            t_max_day,
            t_base)

        cdd_365[day] = cdd

    assert not testing_functions.test_if_minus_value_in_array(cdd_365)

    return cdd_365

def calc_hdd_min_max(t_min, t_max, t_base):
    """Calculate daily hdd with minimum and maximum
    daily temperatures

    Arguments
    ---------
    t_min : array, shape(365)
        Daily minimum temperatures
    t_max : array, shape(365)
        Daily maximum temperatures
    t_base : float
        Base temperature

    Returns
    -------
    hdd_365 : array
        Daily heating degree days
    """
    hdd_365 = np.zeros((365))
    for day in range(365):

        t_min_day = t_min[day]
        t_max_day = t_max[day]

        case_nr = get_meterological_equation_case_hdd(
            t_min_day,
            t_max_day,
            t_base)

        hdd = calc_met_equation_hdd(
            case_nr,
            t_min_day,
            t_max_day,
            t_base)

        hdd_365[day] = hdd

    assert not testing_functions.test_if_minus_value_in_array(hdd_365)
    return hdd_365

def calc_met_equation_hdd(case, t_min, t_max, t_base):
    """Calculate hdd according to Meteorological
    Office equations as outlined in Day (2006): Degree-days: theory and application
    """
    if case == 1:
        hdd = t_base - 0.5 * (t_max + t_min)
    elif case == 2:
        hdd = 0.5 * (t_base - t_min) - 0.25 * (t_max - t_base)
    elif case == 3:
        hdd = 0.25 * (t_base - t_min)
    else:
        hdd = 0

    return hdd

def calc_met_equation_cdd(case, t_min, t_max, t_base):
    """Calculate cdd according to Meteorological
    Office equations as outlined in Day (2006): Degree-days: theory and application

    Arguments
    ---------
    case : int
        Case number
    t_min : float
        Minimum daily temperature
    t_max : float
        Maximum dail temperature
    t_base : float
        Base temperature

    Return
    -------
    case_nr : int
        Case number

    """
    if case == 1:
        cdd = 0.5 * (t_max + t_min) - t_base
    elif case == 2:
        cdd = 0.5 * (t_max - t_base) - 0.25 * (t_base - t_min)
    elif case == 3:
        cdd = 0.25 * (t_max - t_base)
    else:
        cdd = 0

    return cdd

def get_meterological_equation_case_hdd(t_min, t_max, t_base):
    """Calculate case number to calculate hdd with Meteorological Office
    equations

    Arguments
    ---------
    t_min : float
        Minimum daily temperature
    t_max : float
        Maximum dail temperature
    t_base : float
        Base temperature

    Return
    -------
    case_nr : int
        Case number

    """
    # Adding own constrained
    if (t_max - t_base) == (t_base - t_min):
        t_base += 0.0001


    if t_max <= t_base:
        return 1
    elif t_min >= t_base:
        return 4
    elif (t_min < t_base) and ((t_max - t_base) < (t_base - t_min)):
        return 2
    elif (t_max > t_base) and ((t_max - t_base) > (t_base - t_min)):
        return 3

    else:
        raise Exception(
            "Error in calculating methorological office equation case {}  {} {} ".format(t_max, t_min, t_base))

def get_meterological_equation_case_cdd(t_min, t_max, t_base):
    """Calculatease number to calculate cdd with Meteorological
    Office equations as outlined in Day (2006): Degree-days: theory and application

    Arguments
    ---------
    t_min : float
        Minimum daily temperature
    t_max : float
        Maximum dail temperature
    t_base : float
        Base temperature

    Return
    -------
    case_nr : int
        Case number
    """
    # Added own additional constrained because funciton failes if (t_max - t_base) == (t_base - t_min)
    if (t_max - t_base) == (t_base - t_min):
        t_base += 0.0001

    if t_min >= t_base:
        return 1
    elif t_max <= t_base:
        return 4
    elif (t_max > t_base) and ((t_max - t_base) > (t_base - t_min)):
        return 2
    elif (t_min < t_base) and ((t_max - t_base) < (t_base - t_min)):
        return 3
    else:

        raise Exception("Error in calculating methorological office equation case {} {} {}".format(t_min, t_max, t_base))

def calc_hdd(
        t_base,
        temp_yh,
        nr_day_to_av,
        crit_temp_min_max
    ):
    """Calculate effective temperatures and
    heating Degree Days for every day in a year

    Arguments
    ----------
    t_base : int
        Base temperature
    temp_yh : array
        Array containing daily temperatures for each day (shape nr_of_days, 24)
    nr_day_to_av : int
        Number of previous days to average current day
    crit_temp_min_max : bool
        Criteria whether hourly or daily temperature input

    Returns
    -------
    hdd_d : array
        An array containing the Heating Degree Days
        for every day (shape nr_of_days, 1)

    Note
    -----
    This function is optimised for speed. A more intuitive reading is as follows:
    """
    # ---------------------------------------------
    # Average temperature with previous day(s) information
    # ---------------------------------------------
    if not crit_temp_min_max:

        # Calculate effective temperatures
        temp_yh = effective_temps(
            temp_yh,
            nr_day_to_av=nr_day_to_av)

        # Calculate heating degree days
        temp_diff = (t_base - temp_yh) / 24
        temp_diff[temp_diff < 0] = 0
        hdd_d = np.sum(temp_diff, axis=1)

    else:
        # Calculate effective temperatures
        t_min_eff = effective_temps_min_max(
            temp_yh['t_min'],
            nr_day_to_av=nr_day_to_av)

        t_max_eff = effective_temps_min_max(
            temp_yh['t_max'],
            nr_day_to_av=nr_day_to_av)

        # Calculate heating degree days
        hdd_d = calc_hdd_min_max(
            t_min_eff, t_max_eff, t_base)

    return hdd_d

def effective_temps_min_max(temp_365, nr_day_to_av):
    """Calculate effective temperatures for minium and maximum
    temperatures

    Arguments
    ---------
    temp_365 : np.ndarray
        Daily min or max temperatures
    nr_day_to_av : int
        Number of previous days to average current day

    Returns
    -------
    effective_temp_365 : np.array
        Effective daily temperatures
    """
    effective_temp_365 = np.zeros((365), dtype="float")

    # Copy all border days
    for day in range(nr_day_to_av):
        effective_temp_365[day] = temp_365[day]

    # Iterate days in a year
    for day in range(365)[nr_day_to_av:]: # Skip first dates in January

        # Add todays temperature and previous effective temps
        tot_temp = temp_365[day]

        # Add effective temperature of previous day(s)
        for i in range(nr_day_to_av):
            tot_temp = tot_temp + effective_temp_365[day - (i+1)]  # not +=

        effective_temp_365[day] = tot_temp / (nr_day_to_av + 1)

    return effective_temp_365

def effective_temps(temp_yh, nr_day_to_av):
    """Calculate effective temperatures

    Todays temperatures (starting from 2. of January) are averaged with
    yesterdays temperatures. This is done to follow the methodology of
    National Grid (2012): Gas demand forecasting methdolology.

    The effective temperature is half of yesterday’s effective temperature added to half of
    today’s actual temperature. Effective temperature takes into account the previous
    day’s temperature due to consumer behaviour and perception of the weather.

    Arguments
    ---------
    temp_yh : array
        Temperatures for every hour for all modelled days
    nr_day_to_av : int
        Number of previous days to average current day

    Returns
    -------
    effective_temp_yh : array
        Averaged temperatures with temperatures of previous day(s)

    Notes
    ------
    Correlation results (r_value) for base year with different nr_day_to_av.
        nr_day_to_av = 0: 0.807
        nr_day_to_av = 1: 0.840 ()
        nr_day_to_av = 2: 0.865 ()
        nr_day_to_av = 3: 0.878
    """
    effective_temp_yh = np.zeros((365, 24), dtype="float")

    if not hasattr(temp_yh, 'shape'):
        msg = "Expected a numpy array, instead received a {}. Maybe set option " \
              "`crit_temp_min_max = True` in config file."

        raise TypeError(msg.format(type(temp_yh)))

    # Copy all border days
    for day in range(nr_day_to_av):
        effective_temp_yh[day] = temp_yh[day]


    # Iterate days in a year
    for day in range(365)[nr_day_to_av:]: #Skip first dates in January

        # Add todays temperature and previous effective temps
        tot_temp = temp_yh[day]

        # Add effective temperature of previous day(s)
        for i in range(nr_day_to_av):
            tot_temp = tot_temp + effective_temp_yh[day - (i + 1)]  # not +=

        effective_temp_yh[day] = tot_temp / (nr_day_to_av + 1)

    return effective_temp_yh

def calc_weekend_corr_f(model_yeardays_daytype, wkend_factor):
    """Based on the factor 'wkend_factor' daily correction
    factors get calculated for every modelled day in a year.abs

    Arguments
    ---------
    model_yeardays_daytype : list
        Daytype for every modelled days in the base year
    wkend_factor : float
        This factors states how much energy deamnd differs
        depending on whether a day is a weekend or working day
        (e.g. 0.8 means that on a weekend, 80% less demand occurs)

    Returns
    --------
    cdd_weekend_f : array
        Factor to multiply cdd calculations for every
        day in a year
    """
    cdd_weekend_f = np.ones((365))

    for day_nr, day in enumerate(model_yeardays_daytype):
        if day == 'holiday':
            cdd_weekend_f[day_nr] = wkend_factor

    return cdd_weekend_f

def calc_cdd(t_base_cooling, temp_yh, nr_day_to_av, crit_temp_min_max=False):
    """Calculate cooling degree days

    Arguments
    ----------
    t_base_cooling : float
        Base temperature for cooling
    temp_yh : array
        Temperatures for every hour in a year
    nr_day_to_av : array
        Number of days to average temperature

    Return
    ------
    cdd_d : array
        Contains all CDD for every day in a year (nr_of_days, 1)

    Note
    -----
    - For more info see Formual 2.1: Degree-days: theory and application
      https://www.designingbuildings.co.uk/wiki/Cooling_degree_days
    """
    if not crit_temp_min_max:
        # ---------------------------------------------
        # Average temperature with previous day(s) information
        # ---------------------------------------------
        temp_yh = effective_temps(
            temp_yh,
            nr_day_to_av)

        temp_diff = (temp_yh - t_base_cooling) / 24
        temp_diff[temp_diff < 0] = 0
        cdd_d = np.sum(temp_diff, axis=1)

    else:

        # Calculate effective temperatures
        t_min_eff = effective_temps_min_max(
            temp_yh['t_min'],
            nr_day_to_av=nr_day_to_av)

        t_max_eff = effective_temps_min_max(
            temp_yh['t_max'],
            nr_day_to_av=nr_day_to_av)

        # Calculate heating degree days
        cdd_d = calc_cdd_min_max(t_min_eff, t_max_eff, t_base_cooling)

    return cdd_d

def get_hdd_country(
        t_base_heating,
        regions,
        temp_data,
        reg_coord,
        weather_stations,
        crit_temp_min_max=False
    ):
    """Calculate total number of heating degree days in a region for the base year

    Arguments
    ----------
    regions : dict
        Dictionary containing regions
    data : dict
        Dictionary with data
    """
    hdd_regions = {}

    for region in regions:

        # Get closest weather station and temperatures
        closest_station_id = weather_station.get_closest_station(
            reg_coord[region]['latitude'],
            reg_coord[region]['longitude'],
            weather_stations)

        temperatures = temp_data[closest_station_id]

        hdd_reg = calc_hdd(
            t_base_heating,
            temperatures,
            nr_day_to_av=1,
            crit_temp_min_max=crit_temp_min_max)

        hdd_regions[region] = np.sum(hdd_reg)

    return hdd_regions

def get_cdd_country(
        t_base_cooling,
        regions,
        temp_data,
        reg_coord,
        weather_stations,
        crit_temp_min_max=False
    ):
    """Calculate total number of cooling degree days in a
    region for the base year

    Arguments
    ----------
    t_base_cooling : float
        Base temperature assumption
    regions : dict
        Dictionary containing regions
    temp_data : array
        Temperature data
    reg_coord : dict
        Coordinates
    weather_stations : dict
        Weather stations
    """
    cdd_regions = {}

    for region in regions:

        # Get closest weather station and temperatures
        closest_station_id = weather_station.get_closest_station(
            reg_coord[region]['latitude'],
            reg_coord[region]['longitude'],
            weather_stations)

        # Temp data
        temperatures = temp_data[closest_station_id]

        cdd_reg = calc_cdd(
            t_base_cooling,
            temperatures,
            nr_day_to_av=1,
            crit_temp_min_max=crit_temp_min_max)

        cdd_regions[region] = np.sum(cdd_reg)

    return cdd_regions

def calc_reg_hdd(
        temperatures,
        t_base_heating,
        model_yeardays,
        crit_temp_min_max=False
    ):
    """Calculate hdd for every day and daily
    yd shape of heating demand

    Arguments
    ----------
    temperatures : array
        Temperatures
    t_base_heating : float
        Base temperature for heating
    model_yeardays : dict
        Modelled yeardays

    Return
    ------
    hdd_d : array
        Heating degree days for every day in a year (nr_of_days, 1)
    shape_hdd_d : array
        Shape for heating days (only selected modelling days)

    Note
    -----
    - Based on temperatures of a year, the HDD are calculated for every
      day in a year. Based on the sum of all HDD of all days, the relative
      share of heat used for any day is calculated.

    - The Heating Degree Days are calculated based on assumptions of
      the base temperature of the current year.

    - The shape_yd can be calcuated as follows: 1/ np.sum(hdd_d) * hdd_d

    - The diffusion is assumed to be sigmoid
    """
    hdd_d = calc_hdd(
        t_base_heating,
        temperatures,
        nr_day_to_av=1,
        crit_temp_min_max=crit_temp_min_max)

    shape_hdd_d = load_profile.abs_to_rel(hdd_d)

    # Select only modelled yeardays
    shape_hdd_d_selection = shape_hdd_d[model_yeardays]

    return hdd_d, shape_hdd_d_selection

def calc_reg_cdd(
        temperatures,
        t_base_cooling,
        model_yeardays,
        crit_temp_min_max=False
    ):
    """Calculate CDD for every day and daily yd shape of cooling demand

    Arguments
    ----------
    temperatures : array
        Temperatures
    t_base_cooling : array
        Base temperature cooling
    model_yeardays : list
        Modelled yeardays

    Return
    ------
    shape_yd : array
        Fraction of heat for every day. Array-shape: nr_of_days, 1

    Note
    ----
    - Based on temperatures of a year, the CDD are calculated for every
      day in a year. Based on the sum of all CDD of all days, the relative
      share of heat used for any day is calculated.

    - The Cooling Degree Days are calculated based on assumptions of
      the base temperature of the current year.
    """
    if not crit_temp_min_max:
        cdd_d = calc_cdd(
            t_base_cooling,
            temperatures,
            nr_day_to_av=1,
            crit_temp_min_max=crit_temp_min_max)

        shape_cdd_d = load_profile.abs_to_rel(cdd_d)

        # Select only modelled yeardays
        shape_cdd_d_selection = shape_cdd_d[model_yeardays]
        cdd_d_selection = cdd_d[model_yeardays]

        # If no calc_provide flat curve
        if np.sum(cdd_d_selection) == 0:
            shape_cdd_d_selection = np.full(
                (len(model_yeardays)),
                1 / len(model_yeardays))
    else:
        cdd_d = calc_cdd(
            t_base_cooling,
            temperatures,
            nr_day_to_av=1,
            crit_temp_min_max=crit_temp_min_max)

        shape_cdd_d = load_profile.abs_to_rel(cdd_d)

        # Select only modelled yeardays
        shape_cdd_d_selection = shape_cdd_d[model_yeardays]
        cdd_d_selection = cdd_d[model_yeardays]

        # If no calc_provide flat curve
        if np.sum(cdd_d_selection) == 0:
            shape_cdd_d_selection = np.full(
                (len(model_yeardays)),
                1 / len(model_yeardays))

    return cdd_d_selection, shape_cdd_d_selection
